@@ -1,7 +1,7 @@
 /*
  * charconv.c - character code conversion library
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,11 +30,12 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: charconv.c,v 1.50 2004/11/06 12:43:40 shirok Exp $
+ *  $Id: charconv.c,v 1.60 2007/09/13 12:30:25 shirok Exp $
  */
 
 #include <string.h>
 #include <errno.h>
+#define GAUCHE_API_0_9
 #include <gauche.h>
 #include <gauche/extend.h>
 #include "charconv.h"
@@ -51,16 +52,18 @@ typedef struct conv_guess_rec {
 
 /* anchor of the chain of conversion guessing procedures */
 static struct {
+    int dummy;                  /* trick to place this in .data section */
     conv_guess *procs;
     ScmInternalMutex mutex;
-} guess = { NULL };
+} guess = { 1 };
 
 /* anchor of the conversion context used for UCS -> internal char routine */
 static struct {
+    int dummy;                  /* trick to place this in .data section */
     ScmConvInfo *ucs2char;
     ScmConvInfo *char2ucs;
     ScmInternalMutex mutex;
-} ucsconv = { NULL };
+} ucsconv = { 1 };
 
 /*------------------------------------------------------------
  * Query
@@ -135,7 +138,7 @@ static ScmObj conv_name(int dir, ScmPort *remote, const char *from, const char *
     Scm_Printf(SCM_PORT(out), "[conv(%s->%s) %s %S]",
                from, to, (dir == SCM_PORT_INPUT? "from" : "to"),
                Scm_PortName(remote));
-    return Scm_GetOutputStringUnsafe(SCM_PORT(out));
+    return Scm_GetOutputStringUnsafe(SCM_PORT(out), 0);
 }
 
 /*------------------------------------------------------------
@@ -158,7 +161,7 @@ static int conv_input_filler(ScmPort *port, int mincnt)
        inbuf from the last conversion (insize), so we try to fill the
        rest. */
     insize = info->ptr - info->buf;
-    nread = Scm_Getz(info->ptr, info->bufsiz - insize, info->remote);
+    nread = Scm_Getz(info->ptr, info->bufsiz - (int)insize, info->remote);
     if (nread <= 0) {
         /* input reached EOF.  finish the output state */
         if (insize == 0) {
@@ -181,7 +184,7 @@ static int conv_input_filler(ScmPort *port, int mincnt)
             fprintf(stderr, "<= r=%d (reset), out(%p)%d\n",
                     result, outbuf, outroom);
 #endif
-            return result;
+            return (int)result;
         }
     } else {
         insize += nread;
@@ -207,12 +210,12 @@ static int conv_input_filler(ScmPort *port, int mincnt)
            buffer. */
         memmove(info->buf, info->buf+insize-inroom, inroom);
         info->ptr = info->buf + inroom;
-        return info->bufsiz - outroom;
+        return info->bufsiz - (int)outroom;
     } else if (result == ILLEGAL_SEQUENCE) {
         /* it's likely that the input contains invalid sequence. */
-        int cnt = inroom >= 6 ? 6 : inroom;
+        int cnt = inroom >= 6 ? 6 : (int)inroom;
         ScmObj s = Scm_MakeString(info->buf+insize-inroom, cnt, cnt,
-                                  SCM_MAKSTR_COPYING|SCM_MAKSTR_INCOMPLETE);
+                                  SCM_STRING_COPYING|SCM_STRING_INCOMPLETE);
         Scm_Error("invalid character sequence in the input stream: %S ...", s);
     }
 
@@ -222,10 +225,10 @@ static int conv_input_filler(ScmPort *port, int mincnt)
     if (inroom > 0) {
         memmove(info->buf, info->buf+insize-inroom, inroom);
         info->ptr = info->buf + inroom;
-        return info->bufsiz - outroom;
+        return info->bufsiz - (int)outroom;
     } else {
         info->ptr = info->buf;
-        return info->bufsiz - outroom;
+        return info->bufsiz - (int)outroom;
     }
 }
 
@@ -233,6 +236,10 @@ static void conv_input_closer(ScmPort *p)
 {
     ScmConvInfo *info = (ScmConvInfo*)p->src.buf.data;
     jconv_close(info);
+    if (info->ownerp) {
+        Scm_ClosePort(info->remote);
+        info->remoteClosed = TRUE;
+    }
 }
 
 ScmObj Scm_MakeInputConversionPort(ScmPort *fromPort,
@@ -347,11 +354,11 @@ static void conv_output_closer(ScmPort *port)
 
     /* if there's remaining bytes in buf, send them to the remote port. */
     if (info->ptr > info->buf) {
-        Scm_Putz(info->buf, info->ptr - info->buf, info->remote);
+        Scm_Putz(info->buf, (int)(info->ptr - info->buf), info->remote);
         info->ptr = info->buf;
     }
     /* sends out the closing sequence, if any */
-    r = jconv_reset(info, info->buf, info->bufsiz);
+    r = (int)jconv_reset(info, info->buf, info->bufsiz);
 #ifdef JCONV_DEBUG
     fprintf(stderr, "<= r=%d(reset), buf(%p)\n",
             r, info->buf);
@@ -406,14 +413,14 @@ static int conv_output_flusher(ScmPort *port, int cnt, int forcep)
 #else
             /* See the above notes.  We always flush the output buffer
                here, so that we can avoid output buffer overrun. */
-            Scm_Putz(info->buf, outbuf - info->buf, info->remote);
+            Scm_Putz(info->buf, (int)(outbuf - info->buf), info->remote);
             info->ptr = info->buf;
 #endif
-            return len - inroom;
+            return (int)(len - inroom);
         } else if (result == OUTPUT_NOT_ENOUGH) {
             /* Output buffer got full.  Flush it, and continue
                conversion. */
-            Scm_Putz(info->buf, outbuf - info->buf, info->remote);
+            Scm_Putz(info->buf, (int)(outbuf - info->buf), info->remote);
             info->ptr = info->buf;
             continue;
         } else if (result == ILLEGAL_SEQUENCE) {
@@ -428,11 +435,11 @@ static int conv_output_flusher(ScmPort *port, int cnt, int forcep)
 #else
             /* See the above notes.  We always flush the output buffer here,
                so that we can avoid output buffer overrun. */
-            Scm_Putz(info->buf, outbuf - info->buf, info->remote);
+            Scm_Putz(info->buf, (int)(outbuf - info->buf), info->remote);
             info->ptr = info->buf;
 #endif
             if (forcep && len - inroom != cnt) continue;
-            return len - inroom;
+            return (int)(len - inroom);
         }
     }
 }
@@ -598,17 +605,14 @@ static int chartoucs(ScmChar ch)
  * Initialization
  */
 extern void Scm_Init_convlib(ScmModule *module);
+extern void Scm_Init_convaux(void);
 extern void Scm_Init_convguess(void);
-SCM_EXTERN ScmChar (*Scm_UcsToCharHook)(int ucs4);
-SCM_EXTERN int (*Scm_CharToUcsHook)(ScmChar ch);
-SCM_EXTERN ScmPort *(*Scm_CodingAwarePortHook)(ScmPort *src,
-                                               const char *encoding);
 
-void Scm_Init_libcharconv(void)
+SCM_EXTENSION_ENTRY void Scm_Init_libcharconv(void)
 {
     ScmModule *mod;
     SCM_INIT_EXTENSION(charconv);
-    mod = SCM_MODULE(SCM_FIND_MODULE("gauche.charconv", TRUE));
+    mod = SCM_FIND_MODULE("gauche.charconv", SCM_FIND_MODULE_CREATE);
     guess.procs = NULL;
     (void)SCM_INTERNAL_MUTEX_INIT(guess.mutex);
 #if   defined(GAUCHE_CHAR_ENCODING_UTF_8)
@@ -625,7 +629,7 @@ void Scm_Init_libcharconv(void)
     (void)SCM_INTERNAL_MUTEX_INIT(ucsconv.mutex);
     Scm_Init_convguess();
     Scm_Init_convlib(mod);
-    Scm_UcsToCharHook = ucstochar;
-    Scm_CharToUcsHook = chartoucs;
-    Scm_CodingAwarePortHook = coding_aware_conv;
+    Scm_Init_convaux();
+    Scm__InstallCharconvHooks(ucstochar, chartoucs);
+    Scm__InstallCodingAwarePortHook(coding_aware_conv);
 }
