@@ -10,7 +10,7 @@
  *     the allocated memory for random number generator object.
  *   - changed the names of the functions
  *   - added stuff to make it as a Gauche extension module.
- * $Id: mt-random.c,v 1.16 2005/09/10 09:39:12 shirok Exp $
+ * $Id: mt-random.c,v 1.19 2007/09/15 04:00:23 shirok Exp $
  *
  * The original copyright notice follows.
  */
@@ -57,8 +57,8 @@
    email: matumoto@math.keio.ac.jp
 */
 
-#include "mt-random.h"
 #include <math.h>
+#include "mt-random.h"
 
 /* Period parameters */  
 #define M 397
@@ -84,6 +84,25 @@ void Scm_MTInitByUI(ScmMersenneTwister *mt, unsigned long s)
     mt->mti = mti;
 }
 
+void Scm_MTSetSeed(ScmMersenneTwister *mt, ScmObj seed)
+{
+    if (SCM_INTP(seed)) {
+        Scm_MTInitByUI(mt, Scm_GetUInteger(seed));
+    } else if (SCM_BIGNUMP(seed)) {
+        int i; unsigned long s = 0;
+        for (i=0; i<(int)SCM_BIGNUM_SIZE(seed); i++) {
+            s ^= SCM_BIGNUM(seed)->values[i];
+        }
+        Scm_MTInitByUI(mt, s);
+    } else if (SCM_U32VECTORP(seed)) {
+        Scm_MTInitByArray(mt, (ScmInt32*)SCM_U32VECTOR_ELEMENTS(seed),
+                          SCM_U32VECTOR_SIZE(seed));
+    } else {
+        Scm_TypeError("random seed", "an exact integer or u32vector", seed);
+    }
+}
+
+
 /* initialize by an array with array-length */
 /* init_key is the array for initializing keys */
 /* key_length is its length */
@@ -101,7 +120,7 @@ void Scm_MTInitByArray(ScmMersenneTwister *mt,
         mt->mt[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
         i++; j++;
         if (i>=N) { mt->mt[0] = mt->mt[N-1]; i=1; }
-        if (j>=key_length) j=0;
+        if (j>=(int)key_length) j=0;
     }
     for (k=N-1; k; k--) {
         mt->mt[i] = (mt->mt[i] ^ ((mt->mt[i-1] ^ (mt->mt[i-1] >> 30)) * 1566083941UL))
@@ -277,25 +296,16 @@ static ScmObj mt_allocate(ScmClass *klass, ScmObj initargs)
     ScmObj seed = Scm_GetKeyword(key_seed, initargs, SCM_FALSE);
     ScmMersenneTwister *mt;
     
-    if (!SCM_FALSEP(seed) && !SCM_EXACTP(seed) && !SCM_U32VECTORP(seed)) {
-        Scm_Error("seed needs to be an exact integer or a u32vector, but got: %S", seed);
-    }
-    
     mt = SCM_NEW(ScmMersenneTwister);
     SCM_SET_CLASS(mt, &Scm_MersenneTwisterClass);
     mt->mti = N+1;
-    if (SCM_EXACTP(seed)) {
-        Scm_MTInitByUI(mt, Scm_GetUInteger(seed));
-    } else if (SCM_U32VECTORP(seed)) {
-        Scm_MTInitByArray(mt, (ScmInt32*)SCM_U32VECTOR_ELEMENTS(seed),
-                          SCM_U32VECTOR_SIZE(seed));
-    }
+    if (!SCM_FALSEP(seed)) Scm_MTSetSeed(mt, seed);
     return SCM_OBJ(mt);
 }
 
 extern void Scm_Init_mt_lib(ScmModule*);
 
-void Scm_Init_mt_random(void)
+SCM_EXTENSION_ENTRY void Scm_Init_mt_random(void)
 {
     ScmModule *mod = SCM_FIND_MODULE("math.mt-random", SCM_FIND_MODULE_CREATE);
     SCM_INIT_EXTENSION(mtrandom);
