@@ -1,7 +1,7 @@
 /*
  * net.c - network interface
  *
- *   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,13 +30,13 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: net.c,v 1.44 2006/07/05 03:35:54 shirok Exp $
+ *  $Id: net.c,v 1.59 2007/09/30 08:44:21 shirok Exp $
  */
 
 #include "gauche/net.h"
-#include <unistd.h>
 #include <fcntl.h>
 #include <gauche/extend.h>
+#include <gauche/uvector.h>
 
 /*==================================================================
  * Socket
@@ -106,10 +106,10 @@ ScmSocket *make_socket(Socket fd, int type)
 
 ScmObj Scm_MakeSocket(int domain, int type, int protocol)
 {
-    int sock; 
+    intptr_t sock; 
     SCM_SYSCALL(sock, socket(domain, type, protocol));
     if (SOCKET_INVALID(sock)) Scm_SysError("couldn't create socket");
-    return SCM_OBJ(make_socket(sock, type));
+    return SCM_OBJ(make_socket((Socket)sock, type));
 }
 
 ScmObj Scm_SocketShutdown(ScmSocket *s, int how)
@@ -164,11 +164,11 @@ ScmObj Scm_SocketInputPort(ScmSocket *sock, int buffering)
             sock->status < SCM_SOCKET_STATUS_CONNECTED) {
             sockport_err(sock, "input");
         }
-#ifndef __MINGW32__
+#ifndef GAUCHE_WINDOWS
 	infd = sock->fd;
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
 	infd = _open_osfhandle(sock->fd, O_RDONLY);
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
         if (infd == INVALID_SOCKET) sockport_err(sock, "input");
         
         /* NB: I keep the socket itself in the port name, in order to avoid
@@ -191,11 +191,11 @@ ScmObj Scm_SocketOutputPort(ScmSocket *sock, int buffering)
             sock->status < SCM_SOCKET_STATUS_CONNECTED) {
             sockport_err(sock, "output");
         }
-#ifndef __MINGW32__
+#ifndef GAUCHE_WINDOWS
 	outfd = sock->fd;
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
 	outfd = _open_osfhandle(sock->fd, 0);
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
         if (outfd == INVALID_SOCKET) sockport_err(sock, "output");
 
         /* NB: I keep the socket itself in the port name, in order to avoid
@@ -212,14 +212,18 @@ ScmObj Scm_SocketOutputPort(ScmSocket *sock, int buffering)
  * Low-level library
  */
 
+#define CLOSE_CHECK(fd, op, s)                                          \
+    do {                                                                \
+        if (SOCKET_CLOSED(fd)) {                                        \
+            Scm_Error("attempt to %s a closed socket: %S", op, s);      \
+        }                                                               \
+    } while (0)
+        
 ScmObj Scm_SocketBind(ScmSocket *sock, ScmSockAddr *addr)
 {
     ScmSockAddr *naddr;
     int r;
-    
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to bind a closed socket: %S", sock);
-    }
+    CLOSE_CHECK(sock->fd, "bind", sock);
     SCM_SYSCALL(r, bind(sock->fd, &addr->addr, addr->addrlen));
     if (r < 0) {
         Scm_SysError("bind failed to %S", addr);
@@ -241,9 +245,7 @@ ScmObj Scm_SocketBind(ScmSocket *sock, ScmSockAddr *addr)
 ScmObj Scm_SocketListen(ScmSocket *sock, int backlog)
 {
     int r;
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to listen a closed socket: %S", sock);
-    }
+    CLOSE_CHECK(sock->fd, "listen to", sock);
     SCM_SYSCALL(r, listen(sock->fd, backlog));
     if (r < 0) {
         Scm_SysError("listen(2) failed");
@@ -254,15 +256,13 @@ ScmObj Scm_SocketListen(ScmSocket *sock, int backlog)
 
 ScmObj Scm_SocketAccept(ScmSocket *sock)
 {
-    int newfd;
+    Socket newfd;
     struct sockaddr_storage addrbuf;
     socklen_t addrlen = sizeof(addrbuf);
     ScmSocket *newsock;
     ScmClass *addrClass = Scm_ClassOf(SCM_OBJ(sock->address));
-    
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to accept a closed socket: %S", sock);
-    }
+
+    CLOSE_CHECK(sock->fd, "accept from", sock);
     SCM_SYSCALL(newfd, accept(sock->fd, (struct sockaddr*)&addrbuf, &addrlen));
     if (SOCKET_INVALID(newfd)) {
         if (errno == EAGAIN) {
@@ -283,9 +283,7 @@ ScmObj Scm_SocketAccept(ScmSocket *sock)
 ScmObj Scm_SocketConnect(ScmSocket *sock, ScmSockAddr *addr)
 {
     int r;
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to connect a closed socket: %S", sock);
-    }
+    CLOSE_CHECK(sock->fd, "connect to", sock);
     SCM_SYSCALL(r, connect(sock->fd, &addr->addr, addr->addrlen));
     if (r < 0) {
         Scm_SysError("connect failed to %S", addr);
@@ -301,9 +299,7 @@ ScmObj Scm_SocketGetSockName(ScmSocket *sock)
     struct sockaddr_storage addrbuf;
     socklen_t addrlen = sizeof(addrbuf);
 
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to get the name of a closed socket: %S", sock);
-    }
+    CLOSE_CHECK(sock->fd, "get the name of", sock);
     SCM_SYSCALL(r, getsockname(sock->fd, (struct sockaddr*)&addrbuf, &addrlen));
     if (r < 0) {
         Scm_SysError("getsockname(2) failed");
@@ -317,9 +313,7 @@ ScmObj Scm_SocketGetPeerName(ScmSocket *sock)
     struct sockaddr_storage addrbuf;
     socklen_t addrlen = sizeof(addrbuf);
 
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to get the name of a closed socket: %S", sock);
-    }
+    CLOSE_CHECK(sock->fd, "get the name of", sock);
     SCM_SYSCALL(r, getpeername(sock->fd, (struct sockaddr*)&addrbuf, &addrlen));
     if (r < 0) {
         Scm_SysError("getpeername(2) failed");
@@ -327,14 +321,26 @@ ScmObj Scm_SocketGetPeerName(ScmSocket *sock)
     return SCM_OBJ(Scm_MakeSockAddr(NULL, (struct sockaddr*)&addrbuf, addrlen));
 }
 
-ScmObj Scm_SocketSend(ScmSocket *sock, ScmString *msg, int flags)
+static const char *get_message_body(ScmObj msg, u_int *size)
+{
+    if (SCM_UVECTORP(msg)) {
+        *size = Scm_UVectorSizeInBytes(SCM_UVECTOR(msg));
+        return (const char*)SCM_UVECTOR_ELEMENTS(msg);
+    } else if (SCM_STRINGP(msg)) {
+        return Scm_GetStringContent(SCM_STRING(msg), size, NULL, NULL);
+    } else {
+        Scm_TypeError("socket message", "uniform vector or string", msg);
+        *size = 0;              /* dummy */
+        return NULL;
+    }
+}
+
+ScmObj Scm_SocketSend(ScmSocket *sock, ScmObj msg, int flags)
 {
     int r; u_int size;
     const char *cmsg;
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to send to a closed socket: %S", sock);
-    }
-    cmsg = Scm_GetStringContent(msg, &size, NULL, NULL);
+    CLOSE_CHECK(sock->fd, "send to", sock);
+    cmsg = get_message_body(msg, &size);
     SCM_SYSCALL(r, send(sock->fd, cmsg, size, flags));
     if (r < 0) {
         Scm_SysError("send(2) failed");
@@ -342,15 +348,13 @@ ScmObj Scm_SocketSend(ScmSocket *sock, ScmString *msg, int flags)
     return SCM_MAKE_INT(r);
 }
 
-ScmObj Scm_SocketSendTo(ScmSocket *sock, ScmString *msg, ScmSockAddr *to,
+ScmObj Scm_SocketSendTo(ScmSocket *sock, ScmObj msg, ScmSockAddr *to,
                         int flags)
 {
     int r; u_int size;
     const char *cmsg;
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to send to a closed socket: %S", sock);
-    }
-    cmsg = Scm_GetStringContent(msg, &size, NULL, NULL);
+    CLOSE_CHECK(sock->fd, "send to", sock);
+    cmsg = get_message_body(msg, &size);
     SCM_SYSCALL(r, sendto(sock->fd, cmsg, size, flags,
                           &SCM_SOCKADDR(to)->addr, SCM_SOCKADDR(to)->addrlen));
     if (r < 0) {
@@ -363,15 +367,36 @@ ScmObj Scm_SocketRecv(ScmSocket *sock, int bytes, int flags)
 {
     int r;
     char *buf;
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to recv from a closed socket: %S", sock);
-    }
+    CLOSE_CHECK(sock->fd, "recv from", sock);
     buf = SCM_NEW_ATOMIC2(char*, bytes);
     SCM_SYSCALL(r, recv(sock->fd, buf, bytes, flags));
     if (r < 0) {
         Scm_SysError("recv(2) failed");
     }
-    return Scm_MakeString(buf, r, r, SCM_MAKSTR_INCOMPLETE);
+    return Scm_MakeString(buf, r, r, SCM_STRING_INCOMPLETE);
+}
+
+static char *get_message_buffer(ScmUVector *v, u_int *size)
+{
+    if (SCM_UVECTOR_IMMUTABLE_P(v)) {
+        Scm_Error("attempted to use an immutable uniform vector as a buffer");
+    }
+    *size = Scm_UVectorSizeInBytes(v);
+    return (char *)SCM_UVECTOR_ELEMENTS(v);
+}
+
+ScmObj Scm_SocketRecvX(ScmSocket *sock, ScmUVector *buf, int flags)
+{
+    int r;
+    u_int size;
+    char *z;
+    CLOSE_CHECK(sock->fd, "recv from", sock);
+    z = get_message_buffer(buf, &size);
+    SCM_SYSCALL(r, recv(sock->fd, z, size, flags));
+    if (r < 0) {
+        Scm_SysError("recv(2) failed");
+    }
+    return Scm_MakeInteger(r);
 }
 
 ScmObj Scm_SocketRecvFrom(ScmSocket *sock, int bytes, int flags)
@@ -380,19 +405,57 @@ ScmObj Scm_SocketRecvFrom(ScmSocket *sock, int bytes, int flags)
     char *buf;
     struct sockaddr_storage from;
     socklen_t fromlen = sizeof(from);
-    if (SOCKET_CLOSED(sock->fd)) {
-        Scm_Error("attempt to recv from a closed socket: %S", sock);
-    }
+    CLOSE_CHECK(sock->fd, "recv from", sock);
     buf = SCM_NEW_ATOMIC2(char*, bytes);
     SCM_SYSCALL(r, recvfrom(sock->fd, buf, bytes, flags,
                             (struct sockaddr*)&from, &fromlen));
     if (r < 0) {
         Scm_SysError("recvfrom(2) failed");
     }
-    return Scm_Values2(Scm_MakeString(buf, r, r, SCM_MAKSTR_INCOMPLETE),
+    return Scm_Values2(Scm_MakeString(buf, r, r, SCM_STRING_INCOMPLETE),
                        Scm_MakeSockAddr(NULL, (struct sockaddr*)&from, fromlen));
 }
 
+/* ADDRS is a list of socket addresses; if 'from' address type matches
+   one of them, it is used to store the information so that we can avoid
+   allocation.  If no addresses match the incoming type, and ADDRS is
+   a complete list, the information of 'from' is discarded.  If no addresses
+   match the incoming type, and the last cdr of ADDRS is #t (this case
+   includes ADDRS == #t), a new sockaddr is allocated and returned. */
+ScmObj Scm_SocketRecvFromX(ScmSocket *sock, ScmUVector *buf,
+                           ScmObj addrs, int flags)
+{
+    int r;
+    u_int size;
+    char *z;
+    struct sockaddr_storage from;
+    socklen_t fromlen = sizeof(from);
+    ScmObj cp, addr = SCM_FALSE;
+    
+    CLOSE_CHECK(sock->fd, "recv from", sock);
+    z = get_message_buffer(buf, &size);
+    SCM_SYSCALL(r, recvfrom(sock->fd, z, size, flags,
+                            (struct sockaddr*)&from, &fromlen));
+    if (r < 0) {
+        Scm_SysError("recvfrom(2) failed");
+    }
+    SCM_FOR_EACH(cp, addrs) {
+        ScmObj a = SCM_CAR(cp);
+        if (Scm_SockAddrP(a)) {
+            if (SCM_SOCKADDR_FAMILY(a) == from.ss_family) {
+                memcpy(&SCM_SOCKADDR(a)->addr, &from, SCM_SOCKADDR(a)->addrlen);
+                addr = a;
+                break;
+            }
+        }
+    }
+    if (SCM_FALSEP(addr) && SCM_EQ(cp, SCM_TRUE)) {
+        /* Allocate sockaddr */
+        addr = Scm_MakeSockAddr(NULL, (struct sockaddr*)&from, fromlen);
+    }
+    return Scm_Values2(Scm_MakeInteger(r), addr);
+}
+
 /* Low-level setsockopt() and getsockopt() interface. */
 /* for getsockopt(), we need to know the size of the result.
    if rtype > 0, it is used as the size of result buffer and
@@ -402,19 +465,23 @@ ScmObj Scm_SocketRecvFrom(ScmSocket *sock, int bytes, int flags)
 ScmObj Scm_SocketSetOpt(ScmSocket *s, int level, int option, ScmObj value)
 {
     int r = 0;
-    if (SOCKET_CLOSED(s->fd)) {
-        Scm_Error("attempt to set a socket option of a closed socket: %S", s);
-    }
+    CLOSE_CHECK(s->fd, "set a socket option of", s);
     if (SCM_STRINGP(value)) {
         u_int size;
         const char *cvalue = Scm_GetStringContent(SCM_STRING(value), &size,
                                                   NULL, NULL);
         SCM_SYSCALL(r, setsockopt(s->fd, level, option, cvalue, size));
+    } else if (SCM_UVECTORP(value)) {
+        u_int size = Scm_UVectorSizeInBytes(SCM_UVECTOR(value));
+        const char *cvalue = (const char*)SCM_UVECTOR_ELEMENTS(value);
+        SCM_SYSCALL(r, setsockopt(s->fd, level, option, cvalue, size));
     } else if (SCM_INTP(value) || SCM_BIGNUMP(value)) {
         int v = Scm_GetInteger(value);
-        SCM_SYSCALL(r, setsockopt(s->fd, level, option, &v, sizeof(int)));
+        SCM_SYSCALL(r, setsockopt(s->fd, level, option, (void*)&v, sizeof(int)));
     } else {
-        Scm_Error("socket option must be a string or an integer: %S", value);
+        Scm_TypeError("socket option value",
+                      "an integer, a uvector or a string",
+                      value);
     }
     if (r < 0) Scm_SysError("setsockopt failed");
     return SCM_TRUE;
@@ -424,18 +491,16 @@ ScmObj Scm_SocketGetOpt(ScmSocket *s, int level, int option, int rsize)
 {
     int r = 0;
     socklen_t rrsize = rsize;
-    if (SOCKET_CLOSED(s->fd)) {
-        Scm_Error("attempt to get a socket option of a closed socket: %S", s);
-    }
+    CLOSE_CHECK(s->fd, "get a socket option of", s);
     if (rsize > 0) {
         char *buf = SCM_NEW_ATOMIC2(char *, rrsize);
         SCM_SYSCALL(r, getsockopt(s->fd, level, option, buf, &rrsize));
         if (r < 0) Scm_SysError("getsockopt failed");
-        return Scm_MakeString(buf, rrsize, rrsize, SCM_MAKSTR_INCOMPLETE);
+        return Scm_MakeString(buf, rrsize, rrsize, SCM_STRING_INCOMPLETE);
     } else {
         int val;
         rrsize = sizeof(int);
-        SCM_SYSCALL(r, getsockopt(s->fd, level, option, &val, &rrsize));
+        SCM_SYSCALL(r, getsockopt(s->fd, level, option, (void*)&val, &rrsize));
         if (r < 0) Scm_SysError("getsockopt failed");
         return Scm_MakeInteger(val);
     }
@@ -444,26 +509,73 @@ ScmObj Scm_SocketGetOpt(ScmSocket *s, int level, int option, int rsize)
 /*==================================================================
  * Windows/MinGW compatibility layer
  */
-#ifdef __MINGW32__
+#if defined(GAUCHE_WINDOWS)
 
-/* 
- * I should use WSAStringToAddress, but just for the time being...
- */
-int inet_aton(const char *cp, struct in_addr *inp)
+int inet_pton(int af, const char *src, void *dst)
 {
-    unsigned long r = inet_addr(cp);
-    if (r == (unsigned long)-1) {
-	return 0;
-    } else {
-	inp->s_addr = r;
-	return 1;
+    TCHAR *str = SCM_MBS2WCS(src);
+    int r;
+    INT addrsize;
+    struct sockaddr_in sa;
+    struct sockaddr_in6 sa6;
+    
+    switch (af) {
+    case AF_INET:
+        addrsize = (INT)sizeof(sa);
+        r = WSAStringToAddress(str, af, NULL, (LPSOCKADDR)&sa, &addrsize);
+        if (r != 0) return -1;
+        memcpy(dst, &sa.sin_addr, sizeof(struct in_addr));
+        return 1;
+    case AF_INET6:
+        addrsize = (INT)sizeof(sa6);
+        r = WSAStringToAddress(str, af, NULL, (LPSOCKADDR)&sa6, &addrsize);
+        if (r != 0) return -1;
+        memcpy(dst, &sa6.sin6_addr, sizeof(struct in6_addr));
+        return 1;
     }
+    return -1;
 }
 
-/* winsock requires some obscure initialization */
-static WSADATA wsaData;
+const char *inet_ntop(int af, const void *src, char *dst, socklen_t size)
+{
+#define ADDR_MAXLEN 64
+    struct sockaddr_in sa;
+    struct sockaddr_in6 sa6;
+    TCHAR buf[ADDR_MAXLEN];
+    int r;
+    size_t ressize;
+    DWORD tressize = ADDR_MAXLEN-1;
+    const char *res;
+    
+    switch (af) {
+    case AF_INET:
+        memset(&sa, 0, sizeof(sa));
+        sa.sin_family = AF_INET;
+        memcpy(&sa.sin_addr, src, sizeof(struct in_addr));
+        r = WSAAddressToString((LPSOCKADDR)&sa, (DWORD)sizeof(sa), NULL,
+                               buf, &tressize);
+        break;
+    case AF_INET6:
+        memset(&sa6, 0, sizeof(sa6));
+        sa6.sin6_family = AF_INET6;
+        memcpy(&sa6.sin6_addr, src, sizeof(struct in6_addr));
+        r = WSAAddressToString((LPSOCKADDR)&sa6, (DWORD)sizeof(sa6), NULL,
+                               buf, &tressize);
+        break;
+    default:
+        return NULL;
+    }
 
-#endif /*__MINGW32__*/
+    if (r != 0) return NULL;
+    buf[tressize] = 0;
+    res = SCM_WCS2MBS(buf);
+    ressize = strlen(res);
+    if (size <= (int)ressize) return NULL;
+    memcpy(dst, res, ressize+1);
+    return dst;
+#undef ADDR_MAXLEN
+}
+#endif /*GAUCHE_WINDOWS*/
                           
 /*==================================================================
  * Initialization
@@ -474,36 +586,18 @@ extern void Scm_Init_NetDB(ScmModule *mod);
 extern void Scm_Init_netlib(ScmModule *mod);
 extern void Scm_Init_netaux(void);
 
-void Scm_Init_libnet(void)
+
+
+
+SCM_EXTENSION_ENTRY void Scm_Init_libnet(void)
 {
     ScmModule *mod;
 
     SCM_INIT_EXTENSION(net);
     mod = SCM_FIND_MODULE("gauche.net", SCM_FIND_MODULE_CREATE);
-#ifdef __MINGW32__
-    /* NB: I'm supposed to call WSACleanup when application shuts down,
-       or resource leak would happen, according to the Windows document.
-       It's just ridiculous---why can't OS itself clean up the dead process?
-       Anyway, to behave politically correctly, I need a generic callback
-       mechanism for Gauche core to call cleanup routines in Scm_Exit.
-       For now, let us behave like a wild kid.
-    */
-    {
-	int opt;
-	int r = WSAStartup(MAKEWORD(2,2), &wsaData);
-	if (r != 0) {
-	    SetLastError(r);
-	    Scm_SysError("WSAStartup failed");
-	}
-	/* windows voodoo to make _open_osfhandle magic work */
-	opt = SO_SYNCHRONOUS_NONALERT;
-	r = setsockopt(INVALID_SOCKET, SOL_SOCKET,
-		       SO_OPENTYPE, (char*)&opt, sizeof(opt));
-	if (r == SOCKET_ERROR) {
-	    Scm_SysError("winsock initialization failed");
-	}
-    }
-#endif /*__MINGW32__*/
+#ifdef HAVE_IPV6
+    Scm_AddFeature("gauche.net.ipv6", NULL);
+#endif
     Scm_InitStaticClass(&Scm_SocketClass, "<socket>", mod, NULL, 0);
     Scm_Init_NetAddr(mod);
     Scm_Init_NetDB(mod);
