@@ -1,7 +1,7 @@
 ;;;
 ;;; net - network interface
 ;;;  
-;;;   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,11 +30,13 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: net.scm,v 1.33 2006/07/04 10:56:03 shirok Exp $
+;;;  $Id: net.scm,v 1.41 2007/08/30 02:17:59 shirok Exp $
 ;;;
 
 (define-module gauche.net
   (use srfi-1)
+  (use gauche.uvector)
+  (use gauche.sequence)
   (export <socket> make-socket
           |PF_UNSPEC| |PF_UNIX| |PF_INET| |AF_UNSPEC| |AF_UNIX| |AF_INET|
           |SOCK_STREAM| |SOCK_DGRAM| |SOCK_RAW|
@@ -42,7 +44,8 @@
           socket-shutdown socket-close socket-bind socket-connect socket-fd
           socket-listen socket-accept socket-setsockopt socket-getsockopt
           socket-getsockname socket-getpeername
-          socket-send socket-sendto socket-recv socket-recvfrom
+          socket-send socket-sendto
+          socket-recv socket-recv! socket-recvfrom socket-recvfrom!
           <sockaddr> <sockaddr-in> <sockaddr-un> make-sockaddrs
           sockaddr-name sockaddr-family sockaddr-addr sockaddr-port
           make-client-socket make-server-socket make-server-sockets
@@ -51,6 +54,8 @@
           <sys-protoent> sys-getprotobyname sys-getprotobynumber
           <sys-servent> sys-getservbyname sys-getservbyport
           sys-htonl sys-htons sys-ntohl sys-ntohs
+          inet-checksum
+          inet-string->address inet-string->address! inet-address->string
           )
   )
   
@@ -60,6 +65,7 @@
 
 (export-if-defined
  |IPPROTO_IP| |IPPROTO_ICMP| |IPPROTO_TCP| |IPPROTO_UDP| |IPPROTO_IPV6|
+ |IPPROTO_ICMPV6|
  |SOL_SOCKET| |SO_KEEPALIVE| |SO_OOBINLINE| |SO_REUSEADDR| |SO_TYPE|
  |SO_BROADCAST| |SO_SNDBUF| |SO_RCVBUF| |SO_PRIORITY| |SO_ERROR| |SOMAXCONN|
  |SO_DEBUG| |SO_DONTROUTE| |SO_LINGER| |SO_RCVLOWAT| |SO_SNDLOWAT|
@@ -69,7 +75,7 @@
  |MSG_CTRUNC| |MSG_DONTROUTE| |MSG_EOR| |MSG_OOB| |MSG_PEEK|
  |MSG_TRUNC| |MSG_WAITALL|)
 
-;; if ipv6 is supported, these symbols are defiend in the C routine.
+;; if ipv6 is supported, these symbols are defined in the C routine.
 
 (export-if-defined
  |PF_INET6| |AF_INET6|
