@@ -1,7 +1,7 @@
 /*
  * netdb.c - obtain information about network
  *
- *   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: netdb.c,v 1.14 2006/11/04 09:56:59 shirok Exp $
+ *  $Id: netdb.c,v 1.18 2007/09/30 08:44:21 shirok Exp $
  */
 
 #include "gauche/net.h"
@@ -71,7 +71,7 @@ static ScmSysHostent *make_hostent(struct hostent *he)
     SCM_SET_CLASS(entry, SCM_CLASS_SYS_HOSTENT);
     entry->name = SCM_MAKE_STR_COPYING(he->h_name);
     entry->aliases = Scm_CStringArrayToList((const char**)he->h_aliases, -1,
-                                            SCM_MAKSTR_COPYING);
+                                            SCM_STRING_COPYING);
     if (he->h_addrtype == AF_INET) {
         for (p = he->h_addr_list; *p; p++) {
             char buf[50];
@@ -139,7 +139,7 @@ ScmObj Scm_GetHostByAddr(const char *addr, int type)
     if (type != AF_INET) {
         Scm_Error("unsupported address type: %d", type);
     }
-    if (!inet_aton(addr, &iaddr)) {
+    if (inet_pton(AF_INET, addr, &iaddr) <= 0) {
         Scm_Error("bad inet address format: %s", addr);
     }
     
@@ -211,7 +211,7 @@ static ScmClassStaticSlotSpec hostent_slots[] = {
     SCM_CLASS_SLOT_SPEC("name", hostent_name, NULL),
     SCM_CLASS_SLOT_SPEC("aliases", hostent_aliases, NULL),
     SCM_CLASS_SLOT_SPEC("addresses", hostent_addresses, NULL),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*-------------------------------------------------------------
@@ -227,7 +227,7 @@ static ScmSysProtoent *make_protoent(struct protoent *pe)
     SCM_SET_CLASS(entry, SCM_CLASS_SYS_PROTOENT);
     entry->name = SCM_MAKE_STR_COPYING(pe->p_name);
     entry->aliases = Scm_CStringArrayToList((const char **)pe->p_aliases, -1,
-                                            SCM_MAKSTR_COPYING);
+                                            SCM_STRING_COPYING);
     entry->proto = Scm_MakeInteger(pe->p_proto);
     return entry;
 }
@@ -331,7 +331,7 @@ static ScmClassStaticSlotSpec protoent_slots[] = {
     SCM_CLASS_SLOT_SPEC("name", protoent_name, NULL),
     SCM_CLASS_SLOT_SPEC("aliases", protoent_aliases, NULL),
     SCM_CLASS_SLOT_SPEC("proto", protoent_proto, NULL),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*-------------------------------------------------------------
@@ -347,7 +347,7 @@ static ScmSysServent *make_servent(struct servent *se)
     SCM_SET_CLASS(entry, SCM_CLASS_SYS_SERVENT);
     entry->name = SCM_MAKE_STR_COPYING(se->s_name);
     entry->aliases = Scm_CStringArrayToList((const char **)se->s_aliases, -1,
-                                            SCM_MAKSTR_COPYING);
+                                            SCM_STRING_COPYING);
     entry->port = Scm_MakeInteger(ntohs(se->s_port));
     entry->proto = SCM_MAKE_STR_COPYING(se->s_proto);
     return entry;
@@ -460,7 +460,7 @@ static ScmClassStaticSlotSpec servent_slots[] = {
     SCM_CLASS_SLOT_SPEC("aliases", servent_aliases, NULL),
     SCM_CLASS_SLOT_SPEC("port", servent_port, NULL),
     SCM_CLASS_SLOT_SPEC("proto", servent_proto, NULL),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*-------------------------------------------------------------
