@@ -24,7 +24,7 @@
 ;; MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 
 ;;; Modified for Gauche by Shiro Kawai, shiro@acm.org
-;;; $Id: srfi-19-lib.scm,v 1.8 2006/11/08 21:14:50 shirok Exp $
+;;; $Id: srfi-19-lib.scm,v 1.11 2007/03/21 21:28:55 shirok Exp $
 
 (define-module srfi-19
   (use srfi-1)
@@ -84,9 +84,10 @@
 ;;   a different epoch is used.
 
 (define-constant tm:nano 1000000000)
+(define-constant tm:nano/i #i1000000000)
 (define-constant tm:sid  86400)    ; seconds in a day
 (define-constant tm:sihd 43200)    ; seconds in a half day
-(define-constant tm:tai-epoch-in-jd 4881175/2) ; julian day number for 'the epoch'
+(define-constant tm:tai-epoch-in-jd #i4881175/2) ; julian day number for 'the epoch'
 
 ;; each entry is ( tai seconds since epoch . # seconds to subtract for utc )
 ;; note they go higher to lower, and end in 1972.
@@ -175,7 +176,7 @@
          (cpu   (+ (car times) (cadr times)))
          (tick  (list-ref times 4))
          (sec   (quotient cpu tick))
-         (nsec  (* (/ tm:nano tick) (remainder cpu tick))))
+         (nsec  (* (/ tm:nano/i tick) (remainder cpu tick))))
     (make-time type nsec sec)))
 
 (define (tm:current-time-tai type)
@@ -227,7 +228,7 @@
   (if (or (not (and (time? time1) (time? time2)))
 	  (not (eq? (time-type time1) (time-type time2))))
       (errorf "time-difference: incompatible time types: ~s, ~s" time1 time2)
-      (let ( (sec-diff (- (time-second time1) (time-second time2)))
+      (let ( (sec-diff  (- (time-second time1) (time-second time2)))
 	     (nsec-diff (- (time-nanosecond time1) (time-nanosecond time2))) )
 	(set-time-type! time3 time-duration)
 	(if (negative? nsec-diff)
@@ -416,7 +417,7 @@
 
 ;; Gives the seconds/date/month/year
 (define (tm:decode-julian-day-number jdn)
-  (let* ((days (truncate jdn))
+  (let* ((days (inexact->exact (truncate jdn)))
 	 (a (+ days 32044))
 	 (b (quotient (+ (* 4 a) 3) 146097))
 	 (c (- a (quotient (* 146097 b) 4)))
@@ -425,7 +426,7 @@
 	 (m (quotient (+ (* 5 e) 2) 153))
 	 (y (+ (* 100 b) d -4800 (quotient m 10))))
     (values ; seconds date month year
-     (* (- jdn days) tm:sid)
+     (inexact->exact (round (* (- jdn days) tm:sid)))
      (+ e (- (quotient (+ (* 153 m) 2) 5)) 1)
      (+ m 3 (* -12 (quotient m 10)))
      (if (>= 0 y) (- y 1) y))
@@ -649,7 +650,7 @@
        (/ (+ (* hour 60 60)
              (* minute 60)
              second
-             (/ nanosecond tm:nano)
+             (/ nanosecond tm:nano/i)
              (- offset))
           tm:sid))))
 
@@ -660,7 +661,7 @@
 
 (define (time-utc->julian-day time)
   (tm:check-time-type time 'time-utc 'time-utc->julian-day)
-  (+ (/ (+ (time-second time) (/ (time-nanosecond time) tm:nano))
+  (+ (/ (+ (time-second time) (/ (time-nanosecond time) tm:nano/i))
 	tm:sid)
      tm:tai-epoch-in-jd))
 
@@ -672,7 +673,7 @@
   (tm:check-time-type time 'time-tai 'time-tai->julian-day)
   (+ (/ (+ (- (time-second time) 
 	      (tm:leap-second-delta (time-second time)))
-	   (/ (time-nanosecond time) tm:nano))
+	   (/ (time-nanosecond time) tm:nano/i))
 	tm:sid)
      tm:tai-epoch-in-jd))
 
@@ -685,7 +686,7 @@
   (tm:check-time-type time 'time-monotonic 'time-monotonic->julian-day)
   (+ (/ (+ (- (time-second time) 
 	      (tm:leap-second-delta (time-second time)))
-	   (/ (time-nanosecond time) tm:nano))
+	   (/ (time-nanosecond time) tm:nano/i))
 	tm:sid)
      tm:tai-epoch-in-jd))
 
@@ -696,10 +697,10 @@
 
 
 (define (julian-day->time-utc jdn)
-  (let ((nanosecs (* tm:nano tm:sid (- jdn tm:tai-epoch-in-jd))))
+  (let ((nanosecs (* tm:nano/i tm:sid (- jdn tm:tai-epoch-in-jd))))
     (make-time time-utc
                (remainder nanosecs tm:nano)
-               (floor (/ nanosecs tm:nano)))))
+               (floor (/ nanosecs tm:nano/i)))))
 
 (define (julian-day->time-tai jdn)
   (time-utc->time-tai! (julian-day->time-utc jdn)))
@@ -838,7 +839,7 @@
     (#\f . ,(lambda (date pad-with)
               (display (tm:padding (date-second date) pad-with 2))
               (display tm:locale-number-separator)
-              (let1 nanostr (number->string (/ (date-nanosecond date) tm:nano))
+              (let1 nanostr (number->string (/ (date-nanosecond date) tm:nano/i))
                 (cond ((string-index nanostr #\.)
                        => (lambda (i) (display (string-drop nanostr (+ i 1)))))
                       ))))
@@ -908,13 +909,13 @@
     (#\1 . ,(lambda (date pad-with)
               (display (date->string date "~Y-~m-~d"))))
     (#\2 . ,(lambda (date pad-with)
-              (display (date->string date "~k:~M:~S~z"))))
+              (display (date->string date "~H:~M:~S~z"))))
     (#\3 . ,(lambda (date pad-with)
-              (display (date->string date "~k:~M:~S"))))
+              (display (date->string date "~H:~M:~S"))))
     (#\4 . ,(lambda (date pad-with)
-              (display (date->string date "~Y-~m-~dT~k:~M:~S~z"))))
+              (display (date->string date "~Y-~m-~dT~H:~M:~S~z"))))
     (#\5 . ,(lambda (date pad-with)
-              (display (date->string date "~Y-~m-~dT~k:~M:~S"))))
+              (display (date->string date "~Y-~m-~dT~H:~M:~S"))))
     ))
 
 (define (date->string date . maybe-fmtstr)
