@@ -66,6 +66,9 @@
 (test* "list=" #t (list= eq? '(a)))
 (test* "list=" #t (list= eq? '(a) '(a)))
 (test* "list=" #f (list= eq? '(a) '(a b)))
+(test* "list=" #t (list= eq? '(a b) '(a b) '(a b)))
+(test* "list=" #f (list= eq? '(a b) '(a b) '(a b c)))
+(test* "list=" #f (list= eq? '(a b) '(a b) '(a)))
 (test* "list=" #t (list= char-ci=? '(#\a #\b #\z) '(#\A #\B #\Z)))
 (test* "list=" #f (list= char-ci=? '(#\a #\b #\z) '(#\A #\B)))
 (test* "first"  1 (first '(1 2 3 4 5 6 7 8 9 10)))
@@ -617,6 +620,14 @@
        (string-downcase "SPEAK SOFTLY"))
 (test* "string-downcase" "peak"
        (string-downcase "SPEAK SOFTLY" 1 5))
+(test* "string-downcase!" "speak softly"
+       (let ((s (string-copy "SPEAK SOFTLY")))
+         (string-downcase! s)
+         s))
+(test* "string-downcase!" "Speak softly"
+       (let ((s (string-copy "SPEAK SOFTLY")))
+         (string-downcase! s 1)
+         s))
 (test* "string-downcase!" "Speak SOFTLY"
        (let ((s (string-copy "SPEAK SOFTLY")))
          (string-downcase! s 1 5)
@@ -995,7 +1006,8 @@
   (test "julian day number, via date"
         jd
         (lambda ()
-          (date->julian-day (julian-day->date jd)))))
+          (date->julian-day (julian-day->date jd)))
+        =))
 (let1 t0 (make-time time-utc 0 1022191954)
   (test "modified julian day number"
         t0
