@@ -1,7 +1,7 @@
 /*
  * thread.c - Scheme thread API
  *
- *   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: threads.c,v 1.13 2006/11/03 11:11:26 shirok Exp $
+ *  $Id: threads.c,v 1.19 2007/09/16 04:15:59 shirok Exp $
  */
 
 #include <gauche.h>
@@ -39,7 +39,9 @@
 #include <gauche/exception.h>
 #include "threads.h"
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #ifdef HAVE_SCHED_H
 #include <sched.h>
 #endif
@@ -281,7 +283,7 @@ ScmObj Scm_ThreadTerminate(ScmVM *target)
 extern void Scm_Init_mutex(ScmModule*);
 extern void Scm_Init_thrlib(ScmModule*);
 
-void Scm_Init_threads(void)
+SCM_EXTENSION_ENTRY void Scm_Init_threads(void)
 {
     ScmModule *mod = SCM_FIND_MODULE("gauche.threads", SCM_FIND_MODULE_CREATE);
     SCM_INIT_EXTENSION(threads);
