@@ -1,7 +1,7 @@
 ;;;
 ;;; interpolate.scm - string interpolation; to be autoloaded
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: interpolate.scm,v 1.5 2003/07/05 03:29:11 shirok Exp $
+;;;  $Id: interpolate.scm,v 1.10 2007/05/05 04:45:19 shirok Exp $
 ;;;
 
 ;;; #`"The value is ,|foo|." => (string-append "The value is " foo ".")
@@ -56,7 +56,7 @@
              (cond ((eof-object? c2) (write-char c acc) (accum c2 acc))
                    ((char=? c2 #\,)
                     (write-char (read-char) acc) (accum (read-char) acc))
-                   ((char-set-contains? #[\x00-\x20\),\;\[\\\]\{\}\x7f] c2)
+                   ((char-set-contains? #[\x00-\x20\),\;\\\]\}\x7f] c2)
                     (write-char c acc) (accum (read-char) acc))
                    (else
                     (cons (get-output-string acc) (insert))))))
@@ -64,13 +64,12 @@
            (write-char c acc) (accum (read-char) acc))))
   (define (insert)
     (let* ((item
-            (with-error-handler
-             (lambda (e)
-               (error "unmatched parenthesis in interpolating string: ~s" str))
-             (lambda () (read))))
+            (guard (e ((<read-error> e)
+                       (errorf "unmatched parenthesis in interpolating string: ~s" str)))
+              (read)))
            (rest
             (accum (read-char) (open-output-string))))
-      (cons `(,x->string ,item) rest)))
+      (cons `(x->string ,item) rest)))
   (cons 'string-append
         (with-input-from-string str
           (lambda () (accum (read-char) (open-output-string)))))
