@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.package.build - build a package
 ;;;  
-;;;   Copyright (c) 2004-2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2004-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: build.scm,v 1.9 2005/08/31 12:43:29 shirok Exp $
+;;;  $Id: build.scm,v 1.11 2007/03/02 07:39:09 shirok Exp $
 ;;;
 
 ;; *EXPERIMENTAL*
@@ -130,16 +130,16 @@
 ;;;
 
 (define (gauche-package-build uri . opts)
-  (let-keywords* opts ((config  '())
-                       (configure-options #f)
-                       (install-only? :install-only #f)
-                       (dry?         :dry-run #f)
-                       (reconfigure? :reconfigure #f)
-                       (check?       :check #t)
-                       (install?     :install #f)
-                       (clean?       :clean #f)
-                       (sudo-user    :sudo-install #f)
-                       (sudo-pass    :sudo-password #f))
+  (let-keywords opts ((config  '())
+                      (configure-options #f)
+                      (install-only? :install-only #f)
+                      (dry?          :dry-run #f)
+                      (reconfigure?  :reconfigure #f)
+                      (check?        :check #t)
+                      (install?      :install #f)
+                      (clean?        :clean #f)
+                      (sudo-user     :sudo-install #f)
+                      (sudo-pass     :sudo-password #f))
     (parameterize ((dry-run dry?))
       (let* ((tarball   (gauche-package-ensure uri :config config))
              (build-dir (assq-ref config 'build-dir "."))
