@@ -1,7 +1,7 @@
 ;;;
 ;;; parseopt.scm - yet another command-line argument parser
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: parseopt.scm,v 1.8 2004/10/17 10:22:26 shirok Exp $
+;;;  $Id: parseopt.scm,v 1.12 2007/10/02 09:07:36 shirok Exp $
 ;;;
 
 (define-module gauche.parseopt
@@ -59,7 +59,9 @@
 ;; ((<optspec> <help-string>) <handler)
 (define (compose-entry a-spec)
   (let ((optspec (if (pair? (car a-spec)) (caar a-spec) (car a-spec)))
-        (helpstr (and (>= (length+ (car a-spec)) 2) (cadar a-spec)))
+        (helpstr (and-let* ((l (length+ (car a-spec)))
+                            ( (>= l 2 ) ))
+                   (cadar a-spec)))
         (handler (cadr a-spec)))
     (unless (string? optspec)
       (error "option spec must be a string, but got" optspec))
@@ -97,23 +99,22 @@
         (errorf "a number is required for option ~a, but got ~a"
                 (ref optspec 'name) arg)))
   (define (get-real arg)
-    (or (and-let* ((num (string->number arg))
-                   ((real? num)))
-          num)
+    (or (and-let* ([num (string->number arg)]
+                   [ (real? num) ])
+          (exact->inexact num))
         (errorf "a real number is required for option ~a, but got ~a"
                 (ref optspec 'name) arg)))
   (define (get-integer arg)
-    (or (and-let* ((num (string->number arg))
-                   ((exact? num)))
+    (or (and-let* ([num (string->number arg)]
+                   [ (exact? num) ])
           num)
         (errorf "an integer is required for option ~a, but got ~a"
                 (ref optspec 'name) arg)))
   (define (get-sexp arg)
-    (with-error-handler
-        (lambda (e)
-          (errorf "the argument for option ~a is not valid sexp: ~s"
-                  (ref optspec 'name) arg))
-      (lambda () (read-from-string arg))))
+    (guard (e [(<read-error> e)
+               (errorf "the argument for option ~a is not valid sexp: ~s"
+                       (ref optspec 'name) arg)])
+      (read-from-string arg)))
   (define (process-args)
     (let loop ((spec (ref optspec 'args))
                (args args)
