@@ -1,7 +1,7 @@
 ;;;
 ;;; procedure.scm - auxiliary procedure utilities.  to be autoloaded.
 ;;;  
-;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,15 +30,16 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: procedure.scm,v 1.14 2005/08/28 12:59:17 shirok Exp $
+;;;  $Id: procedure.scm,v 1.18 2007/03/02 07:39:08 shirok Exp $
 ;;;
 
 (define-module gauche.procedure
   (use srfi-1)
-  (use srfi-2)
-  (export compose pa$ map$ for-each$ apply$
+  (export compose complement pa$ map$ for-each$ apply$
+          count$ fold$ fold-right$ reduce$ reduce-right$
+          filter$ partition$ remove$ find$ find-tail$
+          any$ every$ delete$ member$ assoc$
           any-pred every-pred
-          let-optionals* let-keywords* get-optional
           arity procedure-arity-includes?
           <arity-at-least> arity-at-least? arity-at-least-value
           case-lambda disasm
@@ -62,6 +63,13 @@
 
 (define (compose$ f) (pa$ compose f))
 
+(define (complement fn)
+  (case (arity fn) ;; some optimization
+    ((0) (lambda () (not (fn))))
+    ((1) (lambda (x) (not (fn x))))
+    ((2) (lambda (x y) (not (fn x y))))
+    (else (lambda args (not (apply fn args))))))
+
 (define (map$ proc)      (pa$ map proc))
 (define (for-each$ proc) (pa$ for-each proc))
 (define (apply$ proc)    (pa$ apply proc))
