@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.test - test framework
 ;;;  
-;;;   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: test.scm,v 1.16 2006/04/06 21:41:00 shirok Exp $
+;;;  $Id: test.scm,v 1.26 2007/08/11 01:15:44 shirok Exp $
 
 ;; Writing your own test
 ;;
@@ -64,9 +64,19 @@
 ;; If stdout is redirected to other than tty, all the verbose logs will go
 ;; there, and only a small amount of messages go to stderr.
 ;;
+;; Some environment variables affect the behavior of the tests.
+;;
+;;  GAUCHE_TEST_REPORT_ERROR  If defined, reports stack trace to stderr
+;;         when the test thunk raises an error (even when it is expected).
+;;         Useful for diagnosis of unexpected errors.
+;;
+;;  GAUCHE_TEST_RECORD_FILE   If defined, names a file the test processes
+;;         keep the total statistics.  Test-end accumulates the stats
+;;         into the named file instead of reporting it out immediately.
 
 (define-module gauche.test
-  (export test test* test-start test-end test-section test-module
+  (export test test* test-start test-end test-section
+          test-module
           *test-error* *test-report-error* test-error? prim-test))
 (select-module gauche.test)
 
@@ -92,11 +102,54 @@
 (define *test-report-error*
   (sys-getenv "GAUCHE_TEST_REPORT_ERROR"))
 
+(define *test-record-file*
+  (sys-getenv "GAUCHE_TEST_RECORD_FILE"))
+
 (define (test-error? obj) (is-a? obj <test-error>))
 
 ;; List of discrepancies
 (define *discrepancy-list* '())
 
+(define *test-counts* (vector 0 0 0 0)) ; total/pass/fail/abort
+
+(define (test-count++) 
+  (vector-set! *test-counts* 0 (+ (vector-ref *test-counts* 0) 1)))
+(define (test-pass++)
+  (vector-set! *test-counts* 1 (+ (vector-ref *test-counts* 1) 1)))
+(define (test-fail++)
+  (vector-set! *test-counts* 2 (+ (vector-ref *test-counts* 2) 1)))
+(define (format-summary)
+  (format "Total: ~5d tests, ~5d passed, ~5d failed, ~5d aborted.\n"
+          (vector-ref *test-counts* 0)
+          (vector-ref *test-counts* 1)
+          (vector-ref *test-counts* 2)
+          (vector-ref *test-counts* 3)))
+(define (read-summary)
+  (when (and (string? *test-record-file*)
+             (file-exists? *test-record-file*))
+    (with-input-from-file *test-record-file*
+      (lambda ()
+        (let ((m (rxmatch #/Total:\s+(\d+)\s+tests,\s+(\d+)\s+passed,\s+(\d+)\s+failed,\s+(\d+)\s+aborted/ (read-line))))
+          (when m
+            (for-each (lambda (i)
+                        (vector-set! *test-counts* i
+                                     (string->number
+                                      (rxmatch-substring m (+ i 1)))))
+                      '(0 1 2 3)))))))
+  ;; We write out aborted+1, in case if the test process fails before test-end
+  ;; For normal case, it will be overwritten by test-end.
+  (let ((orig-abort (vector-ref *test-counts* 3)))
+    (vector-set! *test-counts* 3 (+ orig-abort 1))
+    (write-summary)
+    (vector-set! *test-counts* 3 orig-abort)))
+  
+(define (write-summary)
+  (when (string? *test-record-file*)
+    (receive (p nam) (sys-mkstemp *test-record-file*)
+      (display (format-summary) p)
+      (close-output-port p)
+      (sys-rename nam *test-record-file*))))
+
 ;; Tests ------------------------------------------------------------
 
 ;; Primitive test.  This doesn't use neither with-error-handler nor
@@ -105,13 +158,16 @@
   (let ((cmp (if (pair? compare) (car compare) equal?)))
     (format/ss #t "test ~a, expects ~s ==> " msg expect)
     (flush)
+    (test-count++)
     (let ((r (thunk)))
-      (if (cmp expect r)
-          (format #t "ok\n")
-          (begin
-            (format/ss #t "ERROR: GOT ~S\n" r)
-            (set! *discrepancy-list*
-                  (cons (list msg expect r) *discrepancy-list*))))
+      (cond ((cmp expect r)
+             (format #t "ok\n")
+             (test-pass++))
+            (else
+             (format/ss #t "ERROR: GOT ~S\n" r)
+             (set! *discrepancy-list*
+                   (cons (list msg expect r) *discrepancy-list*))
+             (test-fail++)))
       (flush)
       )))
 
@@ -119,15 +175,14 @@
 (define (test msg expect thunk . compare)
   (apply prim-test msg expect
          (lambda ()
-           (with-error-handler
-               (lambda (e)
-                 (when *test-report-error*
-                   (report-error e))
-                 (make <test-error>
-                   :message (if (is-a? e <message-condition>)
-                              (ref e 'message)
-                              e)))
-             thunk))
+           (guard (e (else
+                      (when *test-report-error*
+                        (report-error e))
+                      (make <test-error>
+                        :message (if (is-a? e <message-condition>)
+                                   (ref e 'message)
+                                   e))))
+             (thunk)))
          compare))
 
 ;; A convenient macro version
@@ -142,7 +197,8 @@
 ;; is excluded from undefined variable check.
 
 (define (test-module module . opts)
-  (let-keywords* opts ((allow-undefined '()))
+  (test-count++)
+  (let-keywords opts ((allow-undefined '()))
     (let1 mod (cond ((module? module) module)
                     ((symbol? module)
                      (or (find-module module)
@@ -191,14 +247,16 @@
                                              (format "~a(~a)" (car z) (cdr z)))
                                            bad-gref)
                                       ", "))))
-        (if (null? report)
-          (format #t "ok\n")
+        (cond
+         ((null? report) (test-pass++) (format #t "ok\n"))
+         (else
+          (test-fail++)
           (let ((s (apply string-append report)))
             (format #t "ERROR: ~a\n" s)
             (set! *discrepancy-list*
                   (cons (list (format #f "bindings in module ~a" (module-name mod))
                               '() s)
-                        *discrepancy-list*))))
+                        *discrepancy-list*)))))
         )
       )))
 
@@ -223,9 +281,15 @@
                 (hash-table-keys (module-table module)))))
 
 (define (closure-grefs closure)
-  (%filter identifier?
-           ((with-module gauche.internal vm-code->list)
-            (closure-code closure))))
+  (define code->list (with-module gauche.internal vm-code->list))
+  (let loop ((r '())
+             (code (code->list (closure-code closure))))
+    (cond ((null? code) r)
+          ((identifier? (car code))
+           (loop (cons (car code) r) (cdr code)))
+          ((is-a? (car code) <compiled-code>)
+           (loop (loop r (code->list (car code))) (cdr code)))
+          (else (loop r (cdr code))))))
 
 (define (dangling-gref? ident closure)
   (and (not ((with-module gauche.internal find-binding)
@@ -245,6 +309,7 @@
     (display s (current-error-port))
     (display pad (current-error-port))
     (flush (current-error-port))
+    (read-summary)
     (when (and (sys-isatty (current-error-port))
                (sys-isatty (current-output-port)))
       (newline (current-error-port))))
@@ -261,17 +326,22 @@
         (o (current-output-port)))
     (define (fmt . args)
       (if (and (sys-isatty e) (sys-isatty o))
-          (apply format/ss o args)
-          (begin (apply format/ss e args)
-                 (apply format/ss o args))))
+        (apply format/ss o args)
+        (begin (apply format/ss e args)
+               (apply format/ss o args))))
     
     (if (null? *discrepancy-list*)
-        (fmt "passed.\n")
-        (begin
-          (fmt "failed.\ndiscrepancies found.  Errors are:\n")
-          (for-each (lambda (r)
-                      (apply fmt "test ~a: expects ~s => got ~s\n" r))
-                    (reverse *discrepancy-list*))))
-    ))
+      (fmt "passed.\n")
+      (begin
+        (fmt "failed.\ndiscrepancies found.  Errors are:\n")
+        (for-each (lambda (r)
+                    (apply fmt "test ~a: expects ~s => got ~s\n" r))
+                  (reverse *discrepancy-list*))))
+
+    (when *test-record-file*
+      (write-summary))
+
+    ;; Returns the number of failed tests.
+    (length *discrepancy-list*)))
 
 (provide "gauche/test")
