@@ -1,7 +1,7 @@
 ;;;
 ;;; mime.scm - parsing MIME (rfc2045) message
 ;;;  
-;;;   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: mime.scm,v 1.11 2004/11/25 22:00:43 shirok Exp $
+;;;  $Id: mime.scm,v 1.13 2007/03/02 07:39:10 shirok Exp $
 ;;;
 
 ;; RFC2045 Multipurpose Internet Mail Extensions (MIME)
@@ -118,15 +118,13 @@
     (#/^=\?([-!#-'*+\w\^-~]+)\?([-!#-'*+\w\^-~]+)\?([!->@-~]+)\?=$/
      (#f charset encoding body)
      (if (ces-conversion-supported? charset #f)
-       (with-error-handler
-           (lambda (e) word) ;; capture illegal encoding
-         (lambda ()
-           (cond ((string-ci=? encoding "q")
-                  (ces-convert (quoted-printable-decode-string body)
-                               charset #f))
-                 ((string-ci=? encoding "b")
-                  (ces-convert (base64-decode-string body) charset #f))
-                 (else word)))) ;; unsupported encoding
+       (guard (e (else word))  ;; capture illegal encoding
+         (cond ((string-ci=? encoding "q")
+                (ces-convert (quoted-printable-decode-string body)
+                             charset #f))
+               ((string-ci=? encoding "b")
+                (ces-convert (base64-decode-string body) charset #f))
+               (else word))) ;; unsupported encoding
        word))
     (else word)))
 
