@@ -1,7 +1,7 @@
 ;;;
 ;;; file/util.scm - filesystem utility functions
 ;;;  
-;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: util.scm,v 1.2 2005/09/14 09:50:48 shirok Exp $
+;;;  $Id: util.scm,v 1.9 2007/10/06 09:59:24 shirok Exp $
 ;;;
 
 ;;; This module provides convenient utility functions to handle
@@ -44,6 +44,7 @@
   (use srfi-11)
   (use srfi-13)
   (use util.list)
+  (use util.match)
   (export current-directory directory-list directory-list2 directory-fold
           home-directory temporary-directory
           make-directory* create-directory* remove-directory* delete-directory*
@@ -65,21 +66,20 @@
 (select-module file.util)
 
 ;; common util
-(define (%stat opts)
-  (if (get-keyword :follow-link? opts #t) sys-stat sys-lstat))
+(define (%stat path follow-link?)
+  (let1 stat (if follow-link? sys-stat sys-lstat)
+    (stat path)))
 
 ;;;=============================================================
 ;;; Directory entries
 
 ;; "current-directory" is found in ChezScheme, MzScheme, etc.
-(define (current-directory . maybe-newdir)
-  (cond ((null? maybe-newdir) (sys-getcwd))
-        ((null? (cdr maybe-newdir))
-         (if (string? (car maybe-newdir))
-             (sys-chdir (car maybe-newdir))
-             (error "directory name should be a string" (car maybe-newdir))))
-        (else
-         (error "too many arguments for current-directory" maybe-newdir))))
+(define (current-directory . opts)
+  (let-optionals* opts ((newdir #f))
+    (match newdir
+      (#f (sys-getcwd))
+      ((? string?) (sys-chdir newdir))
+      (_ (error "directory name should be a string" newdir)))))
 
 (define (home-directory . maybe-user)
   (let-optionals* maybe-user ((user (sys-getuid)))
@@ -89,7 +89,8 @@
       (slot-ref ent 'dir))))
 
 (define (temporary-directory)
-  "/tmp")  ;; would be more smarter
+  (or (sys-getenv "TMPDIR")
+      "/tmp"))
 
 ;; utility for directory-list and directory-list2
 (define (%directory-filter dir pred filter-add-path?)
@@ -98,17 +99,20 @@
       (filter pred (sys-readdir dir))))
 
 (define (%directory-filter-compose opts)
-  (apply every-pred
-         (cond-list ((get-keyword :children? opts #f)
-                     (lambda (e) (not (member (sys-basename e) '("." "..")))))
-                    ((get-keyword :filter opts #f)))))
+  (let-keywords opts ((children? #f)
+                      (filter #f))
+    (apply every-pred
+           (cond-list (children?
+                       (lambda (e) (not (member (sys-basename e) '("." "..")))))
+                      (filter)))))
 
 ;; directory-list DIR &keyword ADD-PATH? FILTER-ADD-PATH? CHILDREN? FILTER
 (define (directory-list dir . opts)
-  (let-keywords* opts ((add-path? #f)
-                       (filter-add-path? #f))
+  (let-keywords opts ((add-path? #f)
+                      (filter-add-path? #f)
+                      . rest)
     (let1 entries
-        (sort (%directory-filter dir (%directory-filter-compose opts)
+        (sort (%directory-filter dir (%directory-filter-compose rest)
                                  filter-add-path?))
       (if add-path?
           (map (cut build-path dir <>) entries)
@@ -116,13 +120,15 @@
 
 ;; directory-list2 DIR &optional ADD-DIR? FILTER-ADD-PATH? CHILDREN? FILTER FOLLOW-LINK?
 (define (directory-list2 dir . opts)
-  (let-keywords* opts ((add-path? #f)
-                       (filter-add-path? #f))
-    (let* ((filters (%directory-filter-compose opts))
-           (selector (let1 stat (%stat opts)
-                       (lambda (e)
-                         (and (file-exists? e)
-                              (eq? (slot-ref (stat e) 'type) 'directory)))))
+  (let-keywords opts ((add-path? #f)
+                      (filter-add-path? #f)
+                      (follow-link? #t)
+                      . rest)
+    (let* ((filters (%directory-filter-compose rest))
+           (selector (lambda (e)
+                       (and (file-exists? e)
+                            (eq? (slot-ref (%stat e follow-link?) 'type)
+                                 'directory))))
            (entries (sort (%directory-filter dir filters filter-add-path?))))
       (if add-path?
           (partition selector
@@ -132,21 +138,25 @@
 
 ;; directory-fold DIR PROC KNIL &keyword LISTER FOLDER FOLLOW-LINK?
 (define (directory-fold dir proc knil . opts)
-  (let* ((follow (get-keyword :follow-link? opts #t))
-         (lister (get-keyword :lister opts
-                              (lambda (path knil)
-                                (directory-list path
-                                                :add-path? #t
-                                                :children? #t))))
-         (folder (get-keyword :folder opts fold))
-         (selector (let1 stat (%stat opts)
-                     (lambda (e)
-                       (and (file-exists? e)
-                            (eq? (slot-ref (stat e) 'type) 'directory))))))
+  (let-keywords opts ((lister (lambda (path knil)
+                                (values
+                                 (directory-list path
+                                                 :add-path? #t
+                                                 :children? #t)
+                                 knil)))
+                      (folder fold)
+                      (follow-link? #t))
+    (define (selector e)
+      (and (file-exists? e)
+           (eq? (slot-ref (%stat e follow-link?) 'type)
+                'directory)))
     (define (rec path knil)
       (if (selector path)
-          (folder rec knil (lister path knil))
-          (proc path knil)))
+        ;; [TODO]: For the backward compatibiliy, we allow LISTER to return
+        ;; only a single value.  Should be removed, probably in 0.9.
+        (receive res (lister path knil)
+          (folder rec (get-optional (cdr res) knil) (car res)))
+        (proc path knil)))
     (rec dir knil)))
 
 ;; mkdir -p
@@ -254,7 +264,7 @@
 
 (define (relative-path? path)
   (cond-expand
-   (gauche-windows
+   (gauche.os.windows
     (not (#/^[\/\\]|^[A-Za-z]:/ path)))
    (else
     (not (#/^\// path)))))
@@ -264,7 +274,7 @@
 
 (define (path-separator)
   (cond-expand
-   (gauche-windows #\\)
+   (gauche.os.windows #\\)
    (else #\/)))
 
 (define (path-extension path)
@@ -283,23 +293,23 @@
     (path-sans-extension path)))
 
 (define (find-file-in-paths name . opts)
-  (let* ((paths (get-keyword :paths opts
-                             (cond ((sys-getenv "PATH")
-                                    => (cut string-split <> 
-					    (cond-expand
-					     (gauche-windows #\;)
-					     (else #\:))))
-                                   (else '()))))
-         (pred  (get-keyword :pred opts file-is-executable?)))
+  (let-keywords opts
+      ((paths (cond ((sys-getenv "PATH")
+                     => (cut string-split <> 
+                             (cond-expand
+                              (gauche.os.windows #\;)
+                              (else #\:))))
+                    (else '())))
+       (pred file-is-executable?))
     (if (absolute-path? name)
-        (and (pred name) name)
-        (let loop ((paths paths))
-          (and (not (null? paths))
-               (let1 p (build-path (car paths) name)
-                 (if (pred p)
-                     p
-                     (loop (cdr paths))))))
-        )))
+      (and (pred name) name)
+      (let loop ((paths paths))
+        (and (not (null? paths))
+             (let1 p (build-path (car paths) name)
+               (if (pred p)
+                 p
+                 (loop (cdr paths))))))
+      )))
 
 ;;;=============================================================
 ;;; File attributes
@@ -311,8 +321,9 @@
   (syntax-rules ()
     ((_ name slot)
      (define (name path . opts)
-       (and (sys-access path |F_OK|)
-            (slot-ref ((%stat opts) path) slot))))))
+       (let-keywords opts ((follow-link? #t))
+         (and (sys-access path |F_OK|)
+              (slot-ref (%stat path follow-link?) slot)))))))
 
 (define-stat-accessor file-type 'type)
 (define-stat-accessor file-perm 'perm)
@@ -349,7 +360,7 @@
 ;; an alias.  We should call Windows API to check for sure in future).
 
 (cond-expand
- (gauche-windows
+ (gauche.os.windows
   (define (%stat-compare s1 s2 f1 f2)
     (let ((p1 (sys-normalize-pathname f1 :absolute #t :canonicalize #t))
 	  (p2 (sys-normalize-pathname f2 :absolute #t :canonicalize #t)))
@@ -452,29 +463,35 @@
 ;;  backup-suffix
 ;;  safe
 ;;  keep-timestamp
+;;  keep-mode
 (define (copy-file src dst . opts)
-  (let* ((if-exists (get-keyword :if-exists opts :error))
-         (backsfx   (get-keyword :backup-suffix opts ".orig"))
-         (safe      (get-keyword :safe opts #f))
-         (keeptime  (get-keyword :keep-timestamp opts #f))
-         (backfile  (string-append dst backsfx))
-         (times     '())
-         (tmpfile   #f)
-         (inport    #f)
-         (outport   #f))
-    (define (rollback)
-      (cond (inport  => close-input-port))
-      (cond (outport => close-output-port))
-      (cond (tmpfile => sys-unlink)))
-    (define (commit)
-      (cond (inport  => close-input-port))
-      (cond (outport => close-output-port))
-      (when tmpfile
-        (when (eq? if-exists :backup) (sys-rename dst backfile))
-        (sys-rename tmpfile dst))
-      (unless (null? times) (apply sys-utime dst times)))
-    (define (open-destination)
-      (if safe
+  (let-keywords opts ((if-exists :error)
+                      (backsfx :backup-suffix ".orig")
+                      (safe #f)
+                      (keeptime :keep-timestamp #f)
+                      (keepmode :keep-mode #f))
+    (let ((backfile  (string-append dst backsfx))
+          (times     '())
+          (tmpfile   #f)
+          (inport    #f)
+          (outport   #f)
+          (default-perm (logand #o666 (lognot (sys-umask)))))
+      (define (rollback)
+        (cond (inport  => close-input-port))
+        (cond (outport => close-output-port))
+        (cond (tmpfile => sys-unlink)))
+      (define (commit)
+        (cond (inport  => close-input-port))
+        (cond (outport => close-output-port))
+        (cond (tmpfile
+               (sys-chmod tmpfile (if keepmode (file-perm src) default-perm))
+               (when (eq? if-exists :backup) (sys-rename dst backfile))
+               (sys-rename tmpfile dst))
+              (else
+               (sys-chmod dst (if keepmode (file-perm src) default-perm))))
+        (unless (null? times) (apply sys-utime dst times)))
+      (define (open-destination)
+        (if safe
           (cond
            ((and (eq? if-exists :error) (file-exists? dst))
             (error "destination file exists" dst))
@@ -492,33 +509,31 @@
            (else
             (set! outport (open-output-file dst :if-exists :supersede)) #t))
           ))
-    (define (do-copy)
-      (with-error-handler
-       (lambda (e) (rollback) (raise e))
-       (lambda ()
-         (set! inport (open-input-file src))
-         (when keeptime
-           (set! times (let1 stat (sys-fstat inport)
-                         (map (cut slot-ref stat <>) '(atime mtime)))))
-         (begin0
-          (and (open-destination)
-               (copy-port inport outport :unit 65536)
-               #t)
-          (commit)))))
-
-    ;; body of copy-file
-    (unless (memq if-exists '(#f :error :supersede :backup))
-      (error "argument for :if-exists must be either :error, :supersede, :backup or #f, but got" if-exists))
-    (when (and (file-exists? src) (file-exists? dst) (file-eqv? src dst))
-      (errorf "source ~s and destination ~s are the same file" src dst))
-    (do-copy)
-    ))
+      (define (do-copy)
+        (guard (e (else (rollback) (raise e)))
+          (set! inport (open-input-file src))
+          (when keeptime
+            (set! times (let1 stat (sys-fstat inport)
+                          (map (cut slot-ref stat <>) '(atime mtime)))))
+          (begin0
+           (and (open-destination)
+                (copy-port inport outport :unit 65536)
+                #t)
+           (commit))))
 
+      ;; body of copy-file
+      (unless (memq if-exists '(#f :error :supersede :backup))
+        (error "argument for :if-exists must be either :error, :supersede, :backup or #f, but got" if-exists))
+      (when (and (file-exists? src) (file-exists? dst) (file-eqv? src dst))
+        (errorf "source ~s and destination ~s are the same file" src dst))
+      (do-copy)
+      )))
+  
 ;; move-file
 ;;  if-exists  - :error :supersede :backup #f
 ;;  backup-suffix
 (define (move-file src dst . opts)
-  (let-keywords* opts
+  (let-keywords opts
       ((if-exists :error)
        (backsfx   :backup-suffix ".orig"))
 
@@ -536,7 +551,7 @@
     (define (do-copying)
       (and (copy-file src dst :if-exists if-exists
                       :backup-suffix backsfx 
-                      :safe #t :keep-timestamp #t)
+                      :safe #t :keep-timestamp #t :keep-mode #t)
            (begin (sys-unlink src) #t)))
     
     ;; body of move-file
@@ -559,17 +574,22 @@
 
 ;; file->string, file->list, file->string-list, file->sexp-list
 ;; shortcuts of port->string etc.
+;; NB: call-with-input-file may pass #f to the proc if :if-does-not-exist #f
+;; is given.  We return #f in such case.
+
+(define (%maybe proc)
+  (lambda (maybe-port) (and maybe-port (proc maybe-port))))
 
 (define (file->string file . opts)
-  (apply call-with-input-file file port->string opts))
+  (apply call-with-input-file file (%maybe port->string) opts))
 
 (define (file->list reader file . opts)
-  (apply call-with-input-file file (cut port->list reader <>) opts))
+  (apply call-with-input-file file (%maybe (cut port->list reader <>)) opts))
 
 (define (file->string-list file . opts)
-  (apply call-with-input-file file (cut port->list read-line <>) opts))
+  (apply call-with-input-file file (%maybe (cut port->list read-line <>)) opts))
 
 (define (file->sexp-list file . opts)
-  (apply call-with-input-file file (cut port->list read <>) opts))
+  (apply call-with-input-file file (%maybe (cut port->list read <>)) opts))
 
 (provide "file/util")
