@@ -1,7 +1,7 @@
 /*
  * char.c - character and character set operations
  *
- *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: char.c,v 1.44 2006/04/29 10:14:08 shirok Exp $
+ *  $Id: char.c,v 1.60 2007/09/07 09:51:12 shirok Exp $
  */
 
 #include <ctype.h>
@@ -110,9 +110,20 @@ ScmChar Scm_IntToDigit(int n, int radix)
 /*
  * Convert UCS4 code <-> character
  * If the native encoding is not utf-8, gauche.charconv module is loaded.
+ * and these pointers are filled.
  */
-ScmChar (*Scm_UcsToCharHook)(int ucs4) = NULL;  /* filled by ext/charconv */
-int (*Scm_CharToUcsHook)(ScmChar ch) = NULL;    /* filled by ext/charconv */
+static ScmChar (*ucs2char_hook)(int ucs4) = NULL;
+static int     (*char2ucs_hook)(ScmChar ch) = NULL;
+
+/* called by gauche.charconv */
+void Scm__InstallCharconvHooks(ScmChar (*u2c)(int), int (*c2u)(ScmChar))
+{
+    ucs2char_hook = u2c;
+    char2ucs_hook = c2u;
+}
+
+ScmChar (*Scm_UcsToCharHook)(int ucs4) = NULL;
+int (*Scm_CharToUcsHook)(ScmChar ch) = NULL;
 
 ScmChar Scm_UcsToChar(int n)
 {
@@ -121,15 +132,16 @@ ScmChar Scm_UcsToChar(int n)
     return (ScmChar)n;
 #elif defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
     if (n < 0x80) return (ScmChar)n; /*ASCII range*/
-    if (Scm_UcsToCharHook == NULL) {
+    if (ucs2char_hook == NULL) {
         /* NB: we don't need mutex here, for the loading of gauche.charconv
            is serialized in Scm_Require. */
-        Scm_Require(SCM_MAKE_STR("gauche/charconv"));
-        if (Scm_UcsToCharHook == NULL) {
+        Scm_Require(SCM_MAKE_STR("gauche/charconv"),
+                    SCM_LOAD_PROPAGATE_ERROR, NULL);
+        if (ucs2char_hook == NULL) {
             Scm_Error("couldn't autoload gauche.charconv");
         }
     }
-    return Scm_UcsToCharHook(n);
+    return ucs2char_hook(n);
 #else
     if (n < 0x100) return (ScmChar)n; /* ISO8859-1 */
     else return SCM_CHAR_INVALID;
@@ -143,15 +155,16 @@ int Scm_CharToUcs(ScmChar ch)
     return (int)ch;
 #elif defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
     if (ch < 0x80) return (int)ch; /*ASCII range*/
-    if (Scm_CharToUcsHook == NULL) {
+    if (char2ucs_hook == NULL) {
         /* NB: we don't need mutex here, for the loading of gauche.charconv
            is serialized in Scm_Require. */
-        Scm_Require(SCM_MAKE_STR("gauche/charconv"));
-        if (Scm_CharToUcsHook == NULL) {
+        Scm_Require(SCM_MAKE_STR("gauche/charconv"),
+                    SCM_LOAD_PROPAGATE_ERROR, NULL);
+        if (char2ucs_hook == NULL) {
             Scm_Error("couldn't autoload gauche.charconv");
         }
     }
-    return Scm_CharToUcsHook(ch);
+    return char2ucs_hook(ch);
 #else
     return (int)ch;             /* ISO8859-1 */
 #endif /*!GAUCHE_CHAR_ENCODING_UTF_8*/
@@ -170,25 +183,10 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_CharSetClass,
                          charset_print, charset_compare, NULL, NULL,
                          SCM_CLASS_DEFAULT_CPL);
 
-/* masks */
-#if SIZEOF_LONG == 4
-#define MASK_BIT_SHIFT  5
-#define MASK_BIT_MASK   0x1f
-#elif SIZEOF_LONG == 8
-#define MASK_BIT_SHIFT  6
-#define MASK_BIT_MASK   0x3f
-#elif SIZEOF_LONG == 16    /* maybe, in some future ... */
-#define MASK_BIT_SHIFT  7
-#define MASK_BIT_MASK   0x7f
-#else
-#error need to set SIZEOF_LONG
-#endif
+#define MASK_ISSET(cs, ch)  SCM_BITS_TEST(cs->small, ch)
+#define MASK_SET(cs, ch)    SCM_BITS_SET(cs->small, ch)
+#define MASK_RESET(cs, ch)  SCM_BITS_RESET(cs->small, ch)
 
-#define MASK_INDEX(ch)       ((ch) >> MASK_BIT_SHIFT)
-#define MASK_BIT(ch)         (1L << ((ch) & MASK_BIT_MASK))
-#define MASK_ISSET(cs, ch)   (!!(cs->mask[MASK_INDEX(ch)] & MASK_BIT(ch)))
-#define MASK_SET(cs, ch)     (cs->mask[MASK_INDEX(ch)] |= MASK_BIT(ch))
-#define MASK_RESET(cs, ch)   (cs->mask[MASK_INDEX(ch)] &= ~MASK_BIT(ch))
 
 /*----------------------------------------------------------------------
  * Printer
@@ -198,23 +196,19 @@ static void charset_print_ch(ScmPort *out, ScmChar ch)
     if (ch < 0x20 || ch == 0x7f) {
         Scm_Printf(out, "\\x%02x", ch);
     } else {
-        char chbuf[SCM_CHAR_MAX_BYTES];
-        int i;
-        SCM_CHAR_PUT(chbuf, ch);
-        for (i=0; i<SCM_CHAR_NBYTES(ch); i++) {
-            Scm_Printf(out, "%c", chbuf[i]);
-        }
+        Scm_Putc(ch, out);
     }
 }
 
 static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
 {
     int prev, code;
-    ScmCharSet *cs = SCM_CHARSET(obj);
-    struct ScmCharSetRange *r;
+    ScmCharSet *cs = SCM_CHAR_SET(obj);
+    ScmTreeIter iter;
+    ScmDictEntry *e;
 
     Scm_Printf(out, "#[");
-    for (prev = -1, code = 0; code < SCM_CHARSET_MASK_CHARS; code++) {
+    for (prev = -1, code = 0; code < SCM_CHAR_SET_SMALL_CHARS; code++) {
         if (MASK_ISSET(cs, code) && prev < 0) {
             charset_print_ch(out, code);
             prev = code;
@@ -233,11 +227,13 @@ static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
             charset_print_ch(out, code-1);
         }
     }
-    for (r = cs->ranges; r; r = r->next) {
-        charset_print_ch(out, r->lo);
-        if (r->hi == r->lo) continue;
-        if (r->hi - r->lo > 2) Scm_Printf(out, "-");
-        charset_print_ch(out, r->hi);
+    Scm_TreeIterInit(&iter, &cs->large, NULL);
+    while ((e = Scm_TreeIterNext(&iter)) != NULL) {
+        charset_print_ch(out, (int)e->key);
+        if (e->value != e->key) {
+            if (e->value - e->key > 2) Scm_Printf(out, "-");
+            charset_print_ch(out, (int)e->value);
+        }
     }
     Scm_Printf(out, "]", obj);
 }
@@ -245,13 +241,19 @@ static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
 /*-----------------------------------------------------------------
  * Constructors
  */
+static int cmp(ScmTreeCore *tc, intptr_t a, intptr_t b)
+{
+    if (a > b) return 1;
+    if (a < b) return -1;
+    return 0;
+}
+
 static ScmCharSet *make_charset(void)
 {
     ScmCharSet *cs = SCM_NEW(ScmCharSet);
-    int i;
     SCM_SET_CLASS(cs, SCM_CLASS_CHARSET);
-    for (i=0; i<SCM_CHARSET_MASK_SIZE; i++) cs->mask[i] = 0;
-    cs->ranges = NULL;
+    Scm_BitsFill(cs->small, 0, SCM_CHAR_SET_SMALL_CHARS, 0);
+    Scm_TreeCoreInit(&cs->large, cmp, NULL);
     return cs;
 }
 
@@ -260,24 +262,11 @@ ScmObj Scm_MakeEmptyCharSet(void)
     return SCM_OBJ(make_charset());
 }
 
-ScmObj Scm_CopyCharSet(ScmCharSet *src)
+ScmObj Scm_CharSetCopy(ScmCharSet *src)
 {
     ScmCharSet *dst = make_charset();
-    struct ScmCharSetRange *rs, *rd = dst->ranges;
-    int i;
-    
-    for (i=0; i<SCM_CHARSET_MASK_SIZE; i++) dst->mask[i] = src->mask[i];
-    for (rs = src->ranges; rs; rs = rs->next) {
-        if (rd == NULL) {
-            rd = dst->ranges = SCM_NEW(struct ScmCharSetRange);
-        } else {
-            rd->next = SCM_NEW(struct ScmCharSetRange);
-            rd = rd->next;
-        }
-        rd->lo = rs->lo;
-        rd->hi = rs->hi;
-    }
-    if (rd) rd->next = NULL;
+    Scm_BitsCopyX(dst->small, 0, src->small, 0, SCM_CHAR_SET_SMALL_CHARS);
+    Scm_TreeCoreCopy(&dst->large, &src->large);
     return SCM_OBJ(dst);
 }
 
@@ -342,8 +331,8 @@ ScmChar Scm_ReadXdigitsFromPort(ScmPort *port, int ndigits,
  */
 static int charset_compare(ScmObj x, ScmObj y, int equalp)
 {
-    ScmCharSet *xx = SCM_CHARSET(x);
-    ScmCharSet *yy = SCM_CHARSET(y);
+    ScmCharSet *xx = SCM_CHAR_SET(x);
+    ScmCharSet *yy = SCM_CHAR_SET(y);
     
     if (equalp) {
         return (Scm_CharSetEq(xx, yy)? 0 : 1);
@@ -358,33 +347,40 @@ static int charset_compare(ScmObj x, ScmObj y, int equalp)
 
 int Scm_CharSetEq(ScmCharSet *x, ScmCharSet *y)
 {
-    int i;
-    struct ScmCharSetRange *rx, *ry;
-    for (i=0; i<SCM_CHARSET_MASK_SIZE; i++)
-        if (x->mask[i] != y->mask[i]) return FALSE;
-    for (rx=x->ranges, ry=y->ranges; rx && ry; rx=rx->next, ry=ry->next) {
-        if (rx->lo != ry->lo || rx->hi != ry->hi) return FALSE;
-    }
-    if (rx || ry) return FALSE;
+    if (!Scm_BitsEqual(x->small, y->small, 0, SCM_CHAR_SET_SMALL_CHARS))
+        return FALSE;
+    if (!Scm_TreeCoreEq(&x->large, &y->large))
+        return FALSE;
     return TRUE;
 }
 
 /* whether x <= y */
 int Scm_CharSetLE(ScmCharSet *x, ScmCharSet *y)
 {
-    int i;
-    struct ScmCharSetRange *rx, *ry;
-    for (i=0; i<SCM_CHARSET_MASK_SIZE; i++)
-        if ((x->mask[i] | y->mask[i]) != y->mask[i]) return FALSE;
-    rx = x->ranges;
-    ry = y->ranges;
-    while (rx && ry) {
-        if (rx->lo < ry->lo) return FALSE;
-        if (rx->lo > ry->hi) { ry = ry->next; continue; }
-        if (rx->hi > ry->hi) return FALSE;
-        rx = rx->next;
+    ScmTreeIter xi;
+    ScmDictEntry *xe, *ye, *yl, *yh;
+    if (!Scm_BitsIncludes(y->small, x->small, 0, SCM_CHAR_SET_SMALL_CHARS))
+        return FALSE;
+    /* For each range of X, check if it is fully contained by a range of Y.
+     *
+     * Case 1:
+     *    xk<---------->xv
+     *    yk<----------------->yv
+     * Case 2:
+     *         xk<---------->xv
+     *    yk<------------------>yv
+     */
+    Scm_TreeIterInit(&xi, &x->large, NULL);
+    for (xe = Scm_TreeIterNext(&xi); xe; xe = Scm_TreeIterNext(&xi)) {
+        ye = Scm_TreeCoreClosestEntries(&y->large, xe->key, &yl, &yh);
+        if (ye) {               /* case 1 */
+            if (ye->value < xe->value) return FALSE;
+        } else if (yl) {        /* case 2 */
+            if (yl->value < xe->value) return FALSE;
+        } else {
+            return FALSE;
+        }
     }
-    if (rx) return FALSE;
     return TRUE;
 }
 
@@ -392,118 +388,85 @@ int Scm_CharSetLE(ScmCharSet *x, ScmCharSet *y)
  * Modification
  */
 
-static struct ScmCharSetRange *newrange(int lo, int hi,
-                                        struct ScmCharSetRange *next)
-{
-    struct ScmCharSetRange *n = SCM_NEW(struct ScmCharSetRange);
-    n->next = next;
-    n->lo = lo;
-    n->hi = hi;
-    return n;
-}
-
 ScmObj Scm_CharSetAddRange(ScmCharSet *cs, ScmChar from, ScmChar to)
 {
-    int i;
-    struct ScmCharSetRange *lo, *lop, *hi;
+    ScmDictEntry *e, *lo, *hi;
     
     if (to < from) return SCM_OBJ(cs);
-    if (from < SCM_CHARSET_MASK_CHARS) {
-        if (to < SCM_CHARSET_MASK_CHARS) {
-            for (i=from; i<=to; i++) MASK_SET(cs, i);
+    if (from < SCM_CHAR_SET_SMALL_CHARS) {
+        if (to < SCM_CHAR_SET_SMALL_CHARS) {
+            Scm_BitsFill(cs->small, (int)from, (int)to+1, TRUE);
             return SCM_OBJ(cs);
         }
-        for (i=from; i<SCM_CHARSET_MASK_CHARS; i++)  MASK_SET(cs, i);
-        from = SCM_CHARSET_MASK_CHARS;
+        Scm_BitsFill(cs->small, (int)from, SCM_CHAR_SET_SMALL_CHARS, TRUE);
+        from = SCM_CHAR_SET_SMALL_CHARS;
     }
-    if (cs->ranges == NULL) {
-        cs->ranges = newrange(from, to, NULL);
-        return SCM_OBJ(cs);
-    }
-    /* Add range.  Ranges are chained from lower character code to higher,
-       without any overlap. */
-    /* First, we scan the ranges so that we'll get...
-        - if FROM is in a range, lo points to it.
-        - if FROM is out of any ranges, lo points to the closest range that
-          is higher than FROM.
-        - if TO is in a range, hi points to the range.
-        - if TO is out of any ranges, hi points to the closest range that
-          is higher than TO. */
-    for (lop = NULL, lo = cs->ranges; lo; lop = lo, lo = lo->next) {
-        if (from <= lo->hi+1) break;
-    }
-    if (!lo) {
-        lop->next = newrange(from, to, NULL);
-        return SCM_OBJ(cs);
-    }
-    for (hi = lo; hi; hi = hi->next) {
-        if (to <= hi->hi) break;
-    }
-    /* Then we insert, extend and/or merge the ranges accordingly. */
-    if (from < lo->lo) { /* FROM extends the LO */
-        if (lo == hi) {
-            if (to < hi->lo-1) {
-                if (lop == NULL) cs->ranges = newrange(from, to, lo);
-                else             lop->next = newrange(from, to, lo);
-            } else {
-                lo->lo = from;
-            }
-        } else if (hi == NULL || to < hi->lo-1) {
-            lo->lo = from;
-            lo->hi = to;
-            lo->next = hi;
+
+    /* Let e have the lower bound. */
+    e = Scm_TreeCoreClosestEntries(&cs->large, from, &lo, &hi);
+    if (!e) {
+        if (!lo || lo->value < from-1) {
+            e = Scm_TreeCoreSearch(&cs->large, from, SCM_DICT_CREATE);
         } else {
-            lo->lo = from;
-            lo->hi = hi->hi;
-            lo->next = hi->next;
-        }
-    } else { /* FROM included in LO */
-        if (lo != hi) {
-            if (hi == NULL || to < hi->lo-1) {
-                lo->hi = to;
-                lo->next = hi;
-            } else {
-                lo->hi = hi->hi;
-                lo->next = hi->next;
-            }
+            e = lo;
         }
     }
-    /* WRITE ME */
+    /* Set up the upper bound.
+       NB: if e is a new entry, e->value is 0. */
+    if (e->value >= to) return SCM_OBJ(cs);
+
+    hi = e;
+    while ((hi = Scm_TreeCoreNextEntry(&cs->large, hi->key)) != NULL) {
+        if (hi->key > to+1) {
+            e->value = to;
+            return SCM_OBJ(cs);
+        }
+        Scm_TreeCoreSearch(&cs->large, hi->key, SCM_DICT_DELETE);
+        if (hi->value > to) {
+            e->value = hi->value;
+            return SCM_OBJ(cs);
+        }
+    }
+    e->value = to;
     return SCM_OBJ(cs);
 }
 
 ScmObj Scm_CharSetAdd(ScmCharSet *dst, ScmCharSet *src)
 {
-    int i;
-    struct ScmCharSetRange *r;
-    for (i=0; i<SCM_CHARSET_MASK_SIZE; i++)
-        dst->mask[i] |= src->mask[i];
-    for (r = src->ranges; r; r = r->next) {
-        Scm_CharSetAddRange(dst, r->lo, r->hi);
+    ScmTreeIter iter;
+    ScmDictEntry *e;
+
+    if (dst == src) return SCM_OBJ(dst);  /* precaution */
+    
+    Scm_BitsOperate(dst->small, SCM_BIT_IOR, dst->small, src->small,
+                    0, SCM_CHAR_SET_SMALL_CHARS);
+    Scm_TreeIterInit(&iter, &src->large, NULL);
+    while ((e = Scm_TreeIterNext(&iter)) != NULL) {
+        Scm_CharSetAddRange(dst, SCM_CHAR(e->key), SCM_CHAR(e->value));
     }
     return SCM_OBJ(dst);
 }
 
 ScmObj Scm_CharSetComplement(ScmCharSet *cs)
 {
-    int i, last;
-    struct ScmCharSetRange *r, *p;
-    for (i=0; i<SCM_CHARSET_MASK_SIZE; i++)
-        cs->mask[i] = ~cs->mask[i];
-    last = SCM_CHARSET_MASK_CHARS;
-    for (p = NULL, r = cs->ranges; r; p = r, r = r->next) {
-        int hi = r->hi+1;
-        if (r->lo != SCM_CHARSET_MASK_CHARS) {
-            r->hi = r->lo - 1;
-            r->lo = last;
-        } else {
-            cs->ranges = r->next;
+    int last;
+    ScmDictEntry *e, *n;
+
+    Scm_BitsOperate(cs->small, SCM_BIT_NOT1, cs->small, NULL,
+                    0, SCM_CHAR_SET_SMALL_CHARS);
+    last = SCM_CHAR_SET_SMALL_CHARS;
+    /* we can't use treeiter, since we modify the tree while traversing it. */
+    while ((e = Scm_TreeCoreNextEntry(&cs->large, last)) != NULL) {
+        Scm_TreeCoreSearch(&cs->large, e->key, SCM_DICT_DELETE);
+        if (last < e->key-1) {
+            n = Scm_TreeCoreSearch(&cs->large, last, SCM_DICT_CREATE);
+            n->value = e->key-1;
         }
-        last = hi;
+        last = (int)e->value+1;
     }
     if (last < SCM_CHAR_MAX) {
-        if (!p) cs->ranges = newrange(last, SCM_CHAR_MAX, NULL);
-        else    p->next = newrange(last, SCM_CHAR_MAX, NULL);
+        n = Scm_TreeCoreSearch(&cs->large, last, SCM_DICT_CREATE);
+        n->value = SCM_CHAR_MAX;
     }
     return SCM_OBJ(cs);
 }
@@ -529,13 +492,12 @@ ScmObj Scm_CharSetCaseFold(ScmCharSet *cs)
 int Scm_CharSetContains(ScmCharSet *cs, ScmChar c)
 {
     if (c < 0) return FALSE;
-    if (c < SCM_CHARSET_MASK_CHARS) return MASK_ISSET(cs, c);
+    if (c < SCM_CHAR_SET_SMALL_CHARS) return MASK_ISSET(cs, c);
     else {
-        struct ScmCharSetRange *r;
-        for (r = cs->ranges; r; r = r->next) {
-            if (r->lo <= c && c <= r->hi) return TRUE;
-        }
-        return FALSE;
+        ScmDictEntry *e, *l, *h;
+        e = Scm_TreeCoreClosestEntries(&cs->large, (int)c, &l, &h);
+        if (e || (l && l->value >= c)) return TRUE;
+        else return FALSE;
     }
 }
 
@@ -548,9 +510,10 @@ ScmObj Scm_CharSetRanges(ScmCharSet *cs)
 {
     ScmObj h = SCM_NIL, t = SCM_NIL, cell;
     int ind, begin = 0, prev = FALSE;
-    struct ScmCharSetRange *r;
-    
-    for (ind = 0; ind < SCM_CHARSET_MASK_CHARS; ind++) {
+    ScmTreeIter iter;
+    ScmDictEntry *e;
+
+    for (ind = 0; ind < SCM_CHAR_SET_SMALL_CHARS; ind++) {
         int bit = MASK_ISSET(cs, ind);
         if (!prev && bit) begin = ind;
         if (prev && !bit) {
@@ -560,21 +523,12 @@ ScmObj Scm_CharSetRanges(ScmCharSet *cs)
         prev = bit;
     }
     if (prev) {
-        if (!cs->ranges || cs->ranges->lo != SCM_CHARSET_MASK_CHARS) {
-            cell = Scm_Cons(SCM_MAKE_INT(begin),
-                            SCM_MAKE_INT(SCM_CHARSET_MASK_CHARS-1));
-            SCM_APPEND1(h, t, cell);
-            r = cs->ranges;
-        } else {
-            cell = Scm_Cons(SCM_MAKE_INT(begin), SCM_MAKE_INT(cs->ranges->hi));
-            SCM_APPEND1(h, t, cell);
-            r = cs->ranges->next;
-        }
-    } else {
-        r = cs->ranges;
+        cell = Scm_Cons(SCM_MAKE_INT(begin), SCM_MAKE_INT(ind-1));
+        SCM_APPEND1(h, t, cell);
     }
-    for (; r; r = r->next) {
-        cell = Scm_Cons(SCM_MAKE_INT(r->lo), SCM_MAKE_INT(r->hi));
+    Scm_TreeIterInit(&iter, &cs->large, NULL);
+    while ((e = Scm_TreeIterNext(&iter)) != NULL) {
+        cell = Scm_Cons(SCM_MAKE_INT(e->key), SCM_MAKE_INT(e->value));
         SCM_APPEND1(h, t, cell);
     }
     return h;
@@ -584,13 +538,16 @@ ScmObj Scm_CharSetRanges(ScmCharSet *cs)
 void Scm_CharSetDump(ScmCharSet *cs, ScmPort *port)
 {
     int i;
-    struct ScmCharSetRange *r;
     Scm_Printf(port, "CharSet %p\nmask:", cs);
-    for (i=0; i<SCM_CHARSET_MASK_SIZE; i++)
-        Scm_Printf(port, "[%08x]", cs->mask[i]);
+    for (i=0; i<SCM_BITS_NUM_WORDS(SCM_CHAR_SET_SMALL_CHARS); i++) {
+#if SIZEOF_LONG == 4
+        Scm_Printf(port, "[%08lx]", cs->small[i]);
+#else
+        Scm_Printf(port, "[%016lx]", cs->small[i]);
+#endif
+    }
     Scm_Printf(port, "\nranges:");
-    for (r=cs->ranges; r; r=r->next)
-        Scm_Printf(port, "(%d-%d)", r->lo, r->hi);
+    Scm_TreeCoreDump(&cs->large, port);
     Scm_Printf(port, "\n");
 }
 #endif /* SCM_DEBUG_HELPER */
@@ -644,7 +601,7 @@ static ScmChar read_charset_xdigits(ScmPort *port, int ndigs, int key)
    whether complement character (caret in the beginning) appeared or not.
    In that case, the returned charset is not complemented. */
 
-static ScmObj read_predef_charset(ScmPort*, ScmObj*);
+static ScmObj read_predef_charset(ScmPort*, ScmObj*, int);
 
 ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
                        int error_p, int bracket_syntax)
@@ -653,7 +610,7 @@ ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
 #define CARET_BEGIN 2
     int begin = REAL_BEGIN, complement = FALSE;
     int lastchar = -1, inrange = FALSE, moreset_complement = FALSE;
-    ScmCharSet *set = SCM_CHARSET(Scm_MakeEmptyCharSet());
+    ScmCharSet *set = SCM_CHAR_SET(Scm_MakeEmptyCharSet());
     ScmObj moreset;
     ScmObj chars = SCM_NIL;
     ScmChar ch = 0;
@@ -713,40 +670,40 @@ ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
                 goto ordchar;
             case 'd':
                 moreset_complement = FALSE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_DIGIT);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
                 break;
             case 'D':
                 moreset_complement = TRUE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_DIGIT);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
                 break;
             case 's':
                 moreset_complement = FALSE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_SPACE);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
                 break;
             case 'S':
                 moreset_complement = TRUE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_SPACE);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
                 break;
             case 'w':
                 moreset_complement = FALSE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_WORD);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_WORD);
                 break;
             case 'W':
                 moreset_complement = TRUE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_WORD);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_WORD);
                 break;
             default:
                 goto ordchar;
             }
             if (moreset_complement) {
-                moreset = Scm_CharSetComplement(SCM_CHARSET(Scm_CopyCharSet(SCM_CHARSET(moreset))));
+                moreset = Scm_CharSetComplement(SCM_CHAR_SET(Scm_CharSetCopy(SCM_CHAR_SET(moreset))));
             }
-            Scm_CharSetAdd(set, SCM_CHARSET(moreset));
+            Scm_CharSetAdd(set, SCM_CHAR_SET(moreset));
             continue;
         case '[':
-            moreset = read_predef_charset(input, &chars);
-            if (!SCM_CHARSETP(moreset)) goto err;
-            Scm_CharSetAdd(set, SCM_CHARSET(moreset));
+            moreset = read_predef_charset(input, &chars, error_p);
+            if (!SCM_CHAR_SET_P(moreset)) goto err;
+            Scm_CharSetAdd(set, SCM_CHAR_SET(moreset));
             continue;
         ordchar:
         default:
@@ -776,16 +733,18 @@ ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
         return SCM_OBJ(set);
     }
   err:
-    if (error_p)
-        Scm_Error("Unclosed bracket in charset syntax [%A",
+    if (error_p) {
+        Scm_Error("Invalid charset syntax [%A",
                   Scm_ListToString(Scm_ReverseX(chars)));
+    }
     return SCM_FALSE;
 }
 
 /* Read posix [:alpha:] etc.  The first '[' is already read.
-   Return #f on error.  Set reverse list of read chars in *chars */
+   Return #f on error if errorp is FALSE.
+   Set reverse list of read chars in *chars */
 #define MAX_CHARSET_NAME_LEN  10
-ScmObj read_predef_charset(ScmPort *input, ScmObj *chars)
+ScmObj read_predef_charset(ScmPort *input, ScmObj *chars, int error_p)
 {
     int i;
     char name[MAX_CHARSET_NAME_LEN];
@@ -796,38 +755,40 @@ ScmObj read_predef_charset(ScmPort *input, ScmObj *chars)
         *chars = Scm_Cons(SCM_MAKE_CHAR(ch), *chars);
         if (!SCM_CHAR_ASCII_P(ch)) break;
         if (ch != ']') {
-            name[i] = ch;
+            name[i] = (char)ch;
             continue;
         }
         if (strncmp(name, ":alnum:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_ALNUM);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_ALNUM);
         } else if (strncmp(name, ":alpha:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_ALPHA);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_ALPHA);
         } else if (strncmp(name, ":blank:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_BLANK);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_BLANK);
         } else if (strncmp(name, ":cntrl:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_CNTRL);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_CNTRL);
         } else if (strncmp(name, ":digit:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_DIGIT);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
         } else if (strncmp(name, ":graph:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_GRAPH);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_GRAPH);
         } else if (strncmp(name, ":lower:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_LOWER);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_LOWER);
         } else if (strncmp(name, ":print:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_PRINT);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_PRINT);
         } else if (strncmp(name, ":punct:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_PUNCT);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_PUNCT);
         } else if (strncmp(name, ":space:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_SPACE);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
         } else if (strncmp(name, ":upper:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_UPPER);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_UPPER);
         } else if (strncmp(name, ":xdigit:", 8) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_XDIGIT);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_XDIGIT);
         } else break;
     }
     /* here we got invalid charset name */
-    name[i] = '\0';
-    Scm_Error("invalid or unsupported POSIX charset '[%s]'", name);
+    if (error_p) {
+        name[i] = '\0';
+        Scm_Error("invalid or unsupported POSIX charset '[%s]'", name);
+    }
     return SCM_FALSE;
 }
 
@@ -839,7 +800,7 @@ ScmObj read_predef_charset(ScmPort *input, ScmObj *chars)
  * ASCII range, that all character sets agree on.
  */
 
-static ScmCharSet *predef_charsets[SCM_CHARSET_NUM_PREDEFINED_SETS] = {NULL};
+static ScmCharSet *predef_charsets[SCM_CHAR_SET_NUM_PREDEFINED_SETS] = {NULL};
 static ScmInternalMutex predef_charsets_mutex;
 
 static void install_charsets(void)
@@ -849,28 +810,28 @@ static void install_charsets(void)
     SCM_INTERNAL_MUTEX_LOCK(predef_charsets_mutex);
 
 #define CS(n)  predef_charsets[n]
-    for (i = 0; i < SCM_CHARSET_NUM_PREDEFINED_SETS; i++) {
-        CS(i) = SCM_CHARSET(Scm_MakeEmptyCharSet());
+    for (i = 0; i < SCM_CHAR_SET_NUM_PREDEFINED_SETS; i++) {
+        CS(i) = SCM_CHAR_SET(Scm_MakeEmptyCharSet());
     }
-    for (code = 0; code < SCM_CHARSET_MASK_CHARS; code++) {
-        if (isalnum(code)) MASK_SET(CS(SCM_CHARSET_ALNUM), code);
-        if (isalpha(code)) MASK_SET(CS(SCM_CHARSET_ALPHA), code);
-        if (iscntrl(code)) MASK_SET(CS(SCM_CHARSET_CNTRL), code);
-        if (isdigit(code)) MASK_SET(CS(SCM_CHARSET_DIGIT), code);
-        if (isgraph(code)) MASK_SET(CS(SCM_CHARSET_GRAPH), code);
-        if (islower(code)) MASK_SET(CS(SCM_CHARSET_LOWER), code);
-        if (isprint(code)) MASK_SET(CS(SCM_CHARSET_PRINT), code);
-        if (ispunct(code)) MASK_SET(CS(SCM_CHARSET_PUNCT), code);
-        if (isspace(code)) MASK_SET(CS(SCM_CHARSET_SPACE), code);
-        if (isupper(code)) MASK_SET(CS(SCM_CHARSET_UPPER), code);
-        if (isxdigit(code)) MASK_SET(CS(SCM_CHARSET_XDIGIT), code);
+    for (code = 0; code < SCM_CHAR_SET_SMALL_CHARS; code++) {
+        if (isalnum(code)) MASK_SET(CS(SCM_CHAR_SET_ALNUM), code);
+        if (isalpha(code)) MASK_SET(CS(SCM_CHAR_SET_ALPHA), code);
+        if (iscntrl(code)) MASK_SET(CS(SCM_CHAR_SET_CNTRL), code);
+        if (isdigit(code)) MASK_SET(CS(SCM_CHAR_SET_DIGIT), code);
+        if (isgraph(code)) MASK_SET(CS(SCM_CHAR_SET_GRAPH), code);
+        if (islower(code)) MASK_SET(CS(SCM_CHAR_SET_LOWER), code);
+        if (isprint(code)) MASK_SET(CS(SCM_CHAR_SET_PRINT), code);
+        if (ispunct(code)) MASK_SET(CS(SCM_CHAR_SET_PUNCT), code);
+        if (isspace(code)) MASK_SET(CS(SCM_CHAR_SET_SPACE), code);
+        if (isupper(code)) MASK_SET(CS(SCM_CHAR_SET_UPPER), code);
+        if (isxdigit(code)) MASK_SET(CS(SCM_CHAR_SET_XDIGIT), code);
         /* Default word constituent chars #[\w].  NB: in future versions,
            a parameter might be introduced to customize this set. */
         if (isalnum(code)||code=='_')
-            MASK_SET(CS(SCM_CHARSET_WORD), code);
+            MASK_SET(CS(SCM_CHAR_SET_WORD), code);
         /* isblank() is not in posix.  for now, I hardcode it. */
         if (code == ' ' || code == '\t')
-            MASK_SET(CS(SCM_CHARSET_BLANK), code);
+            MASK_SET(CS(SCM_CHAR_SET_BLANK), code);
     }
 #undef CS
     SCM_INTERNAL_MUTEX_UNLOCK(predef_charsets_mutex);
@@ -878,7 +839,7 @@ static void install_charsets(void)
 
 ScmObj Scm_GetStandardCharSet(int id)
 {
-    if (id < 0 || id >= SCM_CHARSET_NUM_PREDEFINED_SETS)
+    if (id < 0 || id >= SCM_CHAR_SET_NUM_PREDEFINED_SETS)
         Scm_Error("bad id for predefined charset index: %d", id);
     if (predef_charsets[id] == NULL) {
         install_charsets();
@@ -889,4 +850,15 @@ ScmObj Scm_GetStandardCharSet(int id)
 void Scm__InitChar(void)
 {
     SCM_INTERNAL_MUTEX_INIT(predef_charsets_mutex);
+
+    /* Expose internal charset */
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP)
+    Scm_AddFeature("gauche.ces.eucjp", NULL);
+#elif defined(GAUCHE_CHAR_ENCODING_SJIS)
+    Scm_AddFeature("gauche.ces.sjis", NULL);
+#elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
+    Scm_AddFeature("gauche.ces.utf8", NULL);
+#else
+    Scm_AddFeature("gauche.ces.none", NULL);
+#endif
 }
