@@ -1,7 +1,7 @@
 /*
  * code.c - compiled code builder/handler
  *
- *   Copyright (c) 2005 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2005-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: code.c,v 1.11 2006/11/03 11:11:27 shirok Exp $
+ *  $Id: code.c,v 1.18 2007/08/24 23:55:42 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -93,7 +93,7 @@ static ScmCompiledCode *make_compiled_code(void)
 /*----------------------------------------------------------------------
  * An API to execute statically compiled toplevel code.  *PROVISIONAL*
  */
-static ScmObj execute_toplevels(ScmObj*, int, void*);
+static ScmSubrProc execute_toplevels;
 
 void Scm_VMExecuteToplevels(ScmCompiledCode *cs[])
 {
@@ -105,11 +105,10 @@ static ScmObj execute_toplevels_cc(ScmObj result, void **data)
 {
     ScmCompiledCode **cs = (ScmCompiledCode **)data[0];
     ScmVM *vm;
-
     if (cs[0] == NULL) return SCM_UNDEFINED;
     data[0] = cs+1;
-    Scm_VMPushCC(execute_toplevels_cc, data, 1);
     vm = Scm_VM();
+    Scm_VMPushCC(execute_toplevels_cc, data, 1);
     vm->base = cs[0];
     vm->pc = vm->base->code;
     return SCM_UNDEFINED;
@@ -201,7 +200,7 @@ void Scm_CompiledCodeDump(ScmCompiledCode *cc)
             }
 
             /* Show info */
-            s = Scm_GetOutputStringUnsafe(out);
+            s = Scm_GetOutputStringUnsafe(out, 0);
             if (!SCM_PAIRP(info)) {
                 Scm_Puts(SCM_STRING(s), SCM_CUROUT);
                 Scm_Putc('\n', SCM_CUROUT);
@@ -403,7 +402,7 @@ static void cc_builder_jumpopt(ScmCompiledCode *cc)
     u_int code, i;
     ScmWord *target;
 
-    for (i=0; i<cc->codeSize; i++) {
+    for (i=0; i<(u_int)cc->codeSize; i++) {
         code = SCM_VM_INSN_CODE(*cp); cp++;
         switch (Scm_VMInsnOperandType(code)) {
         case SCM_VM_OPERAND_OBJ:;
@@ -865,12 +864,12 @@ void Scm_CompiledCodeEmit(ScmCompiledCode *cc,
    Addr operand -> integer offset from the beginning of the code */
 ScmObj Scm_CompiledCodeToList(ScmCompiledCode *cc)
 {
-    int i, off;
+    u_int i, off;
     ScmObj h = SCM_NIL, t = SCM_NIL;
     
-    for (i=0; i<cc->codeSize; i++) {
+    for (i=0; i<(u_int)cc->codeSize; i++) {
         ScmWord insn = cc->code[i];
-        int code = SCM_VM_INSN_CODE(insn);
+        u_int code = SCM_VM_INSN_CODE(insn);
         const char *name = Scm_VMInsnName(code);
         
         switch (Scm_VMInsnNumParams(code)) {
@@ -895,11 +894,11 @@ ScmObj Scm_CompiledCodeToList(ScmCompiledCode *cc)
             SCM_APPEND1(h, t, SCM_OBJ(cc->code[++i]));
             break;
         case SCM_VM_OPERAND_ADDR:
-            off = (ScmWord*)cc->code[++i] - cc->code;
+            off = (u_int)((ScmWord*)cc->code[++i] - cc->code);
             SCM_APPEND1(h, t, SCM_MAKE_INT(off));
             break;
         case SCM_VM_OPERAND_OBJ_ADDR:
-            off = (ScmWord*)cc->code[i+2] - cc->code;
+            off = (u_int)((ScmWord*)cc->code[i+2] - cc->code);
             SCM_APPEND(h, t, SCM_LIST2(SCM_OBJ(cc->code[i+1]),
                                        SCM_MAKE_INT(off)));
             i += 2;
@@ -965,7 +964,7 @@ static ScmClassStaticSlotSpec code_slots[] = {
     SCM_CLASS_SLOT_SPEC("size", code_size_get, NULL),
     SCM_CLASS_SLOT_SPEC("max-stack", code_maxstack_get, NULL),
     SCM_CLASS_SLOT_SPEC("intermediate-form", code_iform_get, NULL),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*===========================================================
