@@ -1,7 +1,7 @@
 /*
  * error.c - error handling
  *
- *   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,12 +30,9 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: error.c,v 1.72 2006/11/10 01:22:28 shirok Exp $
+ *  $Id: error.c,v 1.79 2007/08/24 23:55:42 shirok Exp $
  */
 
-#include <errno.h>
-#include <string.h>
-#include <ctype.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/class.h"
@@ -43,6 +40,10 @@
 #include "gauche/vm.h"
 #include "gauche/builtin-syms.h"
 
+#include <errno.h>
+#include <string.h>
+#include <ctype.h>
+
 static void   message_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static ScmObj message_allocate(ScmClass *klass, ScmObj initargs);
 static ScmObj syserror_allocate(ScmClass *klass, ScmObj initargs);
@@ -113,7 +114,7 @@ static void message_set(ScmMessageCondition *obj, ScmObj val)
 
 static ScmClassStaticSlotSpec message_slots[] = {
     SCM_CLASS_SLOT_SPEC("message", message_get, message_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*------------------------------------------------------------
@@ -269,7 +270,6 @@ static void readerror_dummy_set(ScmReadError *obj, ScmObj val)
     /* nothing */
 }
 
-
 static ScmObj porterror_port_get(ScmPortError *obj)
 {
     return obj->port? SCM_OBJ(obj->port) : SCM_FALSE;
@@ -285,12 +285,12 @@ static void porterror_port_set(ScmPortError *obj, ScmObj val)
 
 static ScmClassStaticSlotSpec syserror_slots[] = {
     SCM_CLASS_SLOT_SPEC("errno", syserror_number_get, syserror_number_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 static ScmClassStaticSlotSpec sigerror_slots[] = {
     SCM_CLASS_SLOT_SPEC("signal", sigerror_signal_get, sigerror_signal_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 static ScmClassStaticSlotSpec readerror_slots[] = {
@@ -299,12 +299,12 @@ static ScmClassStaticSlotSpec readerror_slots[] = {
     SCM_CLASS_SLOT_SPEC("column", readerror_dummy_get, readerror_dummy_set),
     SCM_CLASS_SLOT_SPEC("position", readerror_dummy_get, readerror_dummy_set),
     SCM_CLASS_SLOT_SPEC("span", readerror_dummy_get, readerror_dummy_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 static ScmClassStaticSlotSpec porterror_slots[] = {
     SCM_CLASS_SLOT_SPEC("port", porterror_port_get, porterror_port_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*------------------------------------------------------------
@@ -399,7 +399,7 @@ static void   conditions_set(ScmCompoundCondition *obj, ScmObj conds)
 
 static ScmClassStaticSlotSpec compound_slots[] = {
     SCM_CLASS_SLOT_SPEC("%conditions", conditions_get, conditions_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 
@@ -528,11 +528,9 @@ void Scm_Error(const char *msg, ...)
     SCM_VM_RUNTIME_FLAG_SET(vm, SCM_ERROR_BEING_HANDLED);
     
     SCM_UNWIND_PROTECT {
-        ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
         va_start(args, msg);
-        Scm_Vprintf(SCM_PORT(ostr), msg, args, TRUE);
+        e = Scm_MakeError(Scm_Vsprintf(msg, args, TRUE));
         va_end(args);
-        e = Scm_MakeError(Scm_GetOutputString(SCM_PORT(ostr)));
     }
     SCM_WHEN_ERROR {
         /* TODO: should check continuation? */
@@ -551,29 +549,31 @@ void Scm_Error(const char *msg, ...)
 static ScmObj get_syserrmsg(int en)
 {
     ScmObj syserr;
-#ifndef __MINGW32__
+#if !defined(GAUCHE_WINDOWS)
     syserr = SCM_MAKE_STR_COPYING(strerror(en));
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
     LPTSTR msgbuf;
+    const char *xmsgbuf;
     FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
 		  NULL,
 		  en,
 		  0,
 		  (LPTSTR)&msgbuf,
-		  0, NULL);
-    syserr = SCM_MAKE_STR_COPYING(msgbuf);
+                  0, NULL);
+    xmsgbuf = SCM_WCS2MBS(msgbuf);
+    syserr = SCM_MAKE_STR_COPYING(xmsgbuf);
     LocalFree(msgbuf);
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
     return syserr;
 }
 
 static int get_errno(void)
 {
-#ifndef __MINGW32__
+#if !defined(GAUCHE_WINDOWS)
     return errno;
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
     return GetLastError();
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
 }
 
 void Scm_SysError(const char *msg, ...)
@@ -591,7 +591,7 @@ void Scm_SysError(const char *msg, ...)
         va_end(args);
         SCM_PUTZ(": ", -1, ostr);
         SCM_PUTS(syserr, ostr);
-        e = Scm_MakeSystemError(Scm_GetOutputString(SCM_PORT(ostr)), en);
+        e = Scm_MakeSystemError(Scm_GetOutputString(SCM_PORT(ostr), 0), en);
     }
     SCM_WHEN_ERROR {
         /* TODO: should check continuation */
@@ -640,7 +640,7 @@ void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
             SCM_PUTZ(": ", -1, ostr);
             SCM_PUTS(syserr, ostr);
         }
-        smsg = Scm_GetOutputString(SCM_PORT(ostr));
+        smsg = Scm_GetOutputString(SCM_PORT(ostr), 0);
 
         switch (reason) {
         case SCM_PORT_ERROR_INPUT:
@@ -682,12 +682,10 @@ void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
 void Scm_Warn(const char *msg, ...)
 {
     va_list args;
-    ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
     va_start(args, msg);
-    Scm_Vprintf(SCM_PORT(ostr), msg, args, TRUE);
-    va_end(args);
-    Scm_Printf(SCM_CURERR, "WARNING: %A\n", Scm_GetOutputString(SCM_PORT(ostr)));
+    Scm_Printf(SCM_CURERR, "WARNING: %A\n", Scm_Vsprintf(msg, args, TRUE));
     Scm_Flush(SCM_CURERR);
+    va_end(args);
 }
 
 /* format & warn */
@@ -695,7 +693,8 @@ void Scm_FWarn(ScmString *fmt, ScmObj args)
 {
     ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
     Scm_Format(SCM_PORT(ostr), fmt, args, TRUE);
-    Scm_Printf(SCM_CURERR, "WARNING: %A\n", Scm_GetOutputString(SCM_PORT(ostr)));
+    Scm_Printf(SCM_CURERR, "WARNING: %A\n",
+               Scm_GetOutputString(SCM_PORT(ostr), 0));
     Scm_Flush(SCM_CURERR);
 }
 
@@ -751,7 +750,7 @@ ScmObj Scm_RaiseCondition(ScmObj condition_type, ...)
             ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
             Scm_Vprintf(SCM_PORT(ostr), msg, ap, TRUE);
             SCM_APPEND1(argh, argt, SCM_MAKE_KEYWORD("message"));
-            SCM_APPEND1(argh, argt, Scm_GetOutputString(SCM_PORT(ostr)));
+            SCM_APPEND1(argh, argt, Scm_GetOutputString(SCM_PORT(ostr), 0));
             break;
         } else {
             ScmObj arg = va_arg(ap, ScmObj);
