@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche-init.scm - initialize standard environment
 ;;;
-;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: gauche-init.scm,v 1.122 2005/08/25 07:48:24 shirok Exp $
+;;;  $Id: gauche-init.scm,v 1.124 2007/03/07 04:29:14 shirok Exp $
 ;;;
 
 (select-module gauche)
@@ -69,12 +69,3 @@
 ;; for backward compatibility
 (define-module gauche.vm.debugger )
 
-;;
-;; Auxiliary definitions
-;;
-
-(define <exception> <condition>) ;; backward compatibility
-
-(define-reader-ctor 'string-interpolate
-  (lambda (s) (string-interpolate s))) ;;lambda is required to delay loading
-
