@@ -1,7 +1,7 @@
 /*
  * class.h - Gauche object system private header
  *
- *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: class.h,v 1.43 2004/12/18 04:11:13 shirok Exp $
+ *  $Id: class.h,v 1.49 2007/09/13 12:30:28 shirok Exp $
  */
 
 #ifndef GAUCHE_CLASS_H
@@ -74,16 +74,16 @@ struct ScmClassStaticSlotSpecRec {
     ScmSlotAccessor accessor;
 };
 
-#define SCM_CLASS_SLOT_SPEC(name, getter, setter)       \
-    { name, { {SCM_CLASS2TAG(SCM_CLASS_SLOT_ACCESSOR)}, \
-              NULL, NULL,                               \
-              (ScmNativeGetterProc)getter,              \
-              (ScmNativeSetterProc)setter,              \
-              SCM_UNBOUND,                              \
-              SCM_FALSE,                                \
-              SCM_FALSE,                                \
-              TRUE, -1,                                 \
-              SCM_FALSE, SCM_FALSE, SCM_FALSE,          \
+#define SCM_CLASS_SLOT_SPEC(name, getter, setter)               \
+    { name, { {SCM_CLASS_STATIC_TAG(Scm_SlotAccessorClass)},    \
+              NULL, NULL,                                       \
+              (ScmNativeGetterProc)getter,                      \
+              (ScmNativeSetterProc)setter,                      \
+              SCM_UNBOUND,                                      \
+              SCM_FALSE,                                        \
+              SCM_FALSE,                                        \
+              TRUE, -1,                                         \
+              SCM_FALSE, SCM_FALSE, SCM_FALSE,                  \
              } }
 
 #define SCM_CLASS_SLOT_SPEC_END()   { NULL }
