@@ -1,7 +1,7 @@
 /*
  * extend.h - Stuff useful to write Gauche extension
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: extend.h,v 1.5 2003/10/02 09:33:59 shirok Exp $
+ *  $Id: extend.h,v 1.9 2007/09/13 12:30:28 shirok Exp $
  */
 
 #ifndef GAUCHE_EXTEND_H
@@ -44,7 +44,24 @@
 extern "C" {
 #endif
 
-#ifndef __CYGWIN__
+#if defined(__CYGWIN__)
+/* Cygwin's loader rearranges placement of bss, so Scm__bssstart_ and
+   Scm__bssend_ are no longer useful to find bss area.  It defines
+   _bss_start__ and _bss_end__ for each DLLs so we can use it.
+   Note that these symbols are declared in gc.h. */
+#define SCM_INIT_EXTENSION(name)                                        \
+    do {                                                                \
+       extern void *SCM_CPP_CAT(Scm__datastart_, name);                 \
+       extern void *SCM_CPP_CAT(Scm__dataend_, name);                   \
+       Scm_RegisterDL((void*)&SCM_CPP_CAT(Scm__datastart_, name),       \
+                      (void*)&SCM_CPP_CAT(Scm__dataend_, name),         \
+                      (void*)&_bss_start__,                             \
+                      (void*)&_bss_end__);                              \
+    } while (0)
+#elif defined(GAUCHE_WINDOWS)
+/* Um... we count on Boehm GC windows port to find dll data area properly. */
+#define SCM_INIT_EXTENSION(name)  /*nothing*/
+#else
 #define SCM_INIT_EXTENSION(name)                                        \
    do {                                                                 \
        extern void *SCM_CPP_CAT(Scm__datastart_, name);                 \
@@ -56,20 +73,14 @@ extern "C" {
                       (void*)&SCM_CPP_CAT(Scm__bssstart_, name),        \
                       (void*)&SCM_CPP_CAT(Scm__bssend_, name));         \
    } while (0)
-#else  /* __CYGWIN__ */
-#define SCM_INIT_EXTENSION(name)                                        \
-   do {                                                                 \
-       extern void *SCM_CPP_CAT(Scm__datastart_, name);                 \
-       extern void *SCM_CPP_CAT(Scm__dataend_, name);                   \
-       extern void *_bss_start__;                  \
-       extern void *_bss_end__;                    \
-       Scm_RegisterDL((void*)&SCM_CPP_CAT(Scm__datastart_, name),       \
-                      (void*)&SCM_CPP_CAT(Scm__dataend_, name),         \
-                      (void*)&_bss_start__,        \
-                      (void*)&_bss_end__);         \
-   } while (0)
-#endif /* __CYGWIN__ */
+#endif /* !__CYGWIN__ && !GAUCHE_WINDOWS*/
 
+/* MSVC needs dllexport magic */
+#if defined(GAUCHE_WINDOWS)
+#define SCM_EXTENSION_ENTRY __declspec(dllexport)
+#else
+#define SCM_EXTENSION_ENTRY /*none*/
+#endif
 
 #ifdef __cplusplus
 }
